#!/usr/bin/python3
"""
    GeckoConfigStruct - A class to manage the ConfigStruct for 'InXM v8'
"""

from . import (
    GeckoByteStructAccessor,
    GeckoWordStructAccessor,
    GeckoTimeStructAccessor,
    GeckoBoolStructAccessor,
    GeckoEnumStructAccessor,
    GeckoTempStructAccessor,
)

# Constants for this class
ACCPQI = "".join(
    chr(c) for c in [65, 108, 119, 97, 121, 115, 69, 110, 97, 98, 108, 101, 100]
)
ACQFFT = "".join(
    chr(c)
    for c in [
        70,
        97,
        110,
        65,
        99,
        116,
        105,
        118,
        101,
        68,
        101,
        108,
        97,
        121,
        65,
        102,
        116,
        101,
        114,
        80,
        117,
        109,
        112,
        82,
        117,
        110,
    ]
)
AFIKJP = "".join(
    chr(c) for c in [83, 101, 108, 102, 67, 108, 101, 97, 110, 77, 115, 103]
)
AHEOCT = "".join(chr(c) for c in [79, 117, 116, 49, 66, 67, 117, 114])
AIIDNI = "".join(
    chr(c)
    for c in [
        70,
        97,
        110,
        65,
        99,
        116,
        105,
        118,
        101,
        79,
        110,
        65,
        109,
        98,
        105,
        97,
        110,
        116,
        79,
        84,
    ]
)
AKQXPI = "".join(chr(c) for c in [79, 78, 90, 69, 78])
AMJMAO = 15
AOAWBS = "".join(chr(c) for c in [70])
AONPYY = "".join(chr(c) for c in [80, 117, 114, 103, 101, 79, 110, 108, 121])
ASSAKQ = "".join(chr(c) for c in [84, 118, 76, 105, 102, 116])
AWBSIR = "".join(chr(c) for c in [72, 101, 97, 116, 101, 114, 80, 117, 109, 112])
BDJQRJ = "".join(chr(c) for c in [70, 105, 108, 116, 83, 116, 97, 114, 116])
BFEGZU = "".join(chr(c) for c in [79, 117, 116, 73, 79, 50])
BIAMJM = 45
BJEUTO = "".join(
    chr(c)
    for c in [66, 108, 111, 119, 101, 114, 75, 101, 121, 79, 112, 116, 105, 111, 110]
)
BLKXSJ = 49
BMJVHF = "".join(chr(c) for c in [79, 117, 116, 49, 65])
BQFYLJ = "".join(
    chr(c) for c in [69, 120, 116, 80, 114, 111, 98, 101, 69, 110, 97, 98, 108, 101]
)
BQNRXC = "".join(
    chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 55, 78, 98, 87, 101, 101, 107]
)
BQSNQL = 88
BSIRYX = "".join(
    chr(c)
    for c in [72, 101, 97, 116, 82, 101, 115, 116, 114, 105, 99, 116, 105, 111, 110]
)
BSKSOK = 110
BSSUHB = "".join(chr(c) for c in [80, 117, 109, 112, 51, 65, 115, 86, 83, 80])
BVWVUB = "".join(
    chr(c) for c in [86, 83, 80, 83, 112, 101, 101, 100, 76, 101, 118, 101, 108, 48]
)
BXTIAC = "".join(
    chr(c)
    for c in [70, 97, 110, 83, 101, 116, 65, 79, 84, 84, 101, 109, 112, 65, 100, 99]
)
BXYBQS = "".join(chr(c) for c in [66, 114, 83, 101, 116, 73, 110, 100, 101, 120])
BYGDSB = 10
CCPQIP = "".join(
    chr(c) for c in [87, 105, 116, 104, 83, 80, 79, 118, 101, 114, 57, 53, 70]
)
CHWDAF = "".join(chr(c) for c in [65, 109, 80, 109])
CQBMJV = 0
CQFFTT = 19
CRTFMN = 63
CTHBSK = "".join(chr(c) for c in [79, 117, 116, 50, 66, 67, 117, 114])
CVYYPI = "".join(chr(c) for c in [80, 52, 72])
CWAONP = "".join(
    chr(c) for c in [70, 105, 108, 116, 73, 110, 116, 101, 114, 102, 97, 99, 101]
)
CXQIEF = 95
DAFIKJ = "".join(
    chr(c) for c in [65, 109, 98, 105, 97, 110, 116, 79, 72, 69, 110, 97, 98, 108, 101]
)
DJQRJJ = 28
DNIBXT = 7
DNQGVU = 70
DQLAII = "".join(
    chr(c) for c in [70, 111, 114, 99, 101, 77, 97, 120, 80, 111, 119, 101, 114]
)
DSBDJQ = "".join(chr(c) for c in [70, 105, 108, 116, 70, 114, 101, 113])
DUBSSU = "".join(chr(c) for c in [80, 117, 109, 112, 49, 65, 115, 86, 83, 80])
ECVYYP = "".join(chr(c) for c in [80, 51, 76])
EFJTAC = 4
EFXQGL = 97
EJNIBX = "".join(chr(c) for c in [78, 98, 80, 104, 97, 115, 101, 115])
EKCWAO = "".join(
    chr(c) for c in [81, 117, 105, 101, 116, 84, 105, 109, 101, 79, 117, 116]
)
EKVKZI = 60
ELHBQN = 79
EOCTHB = "".join(chr(c) for c in [79, 117, 116, 50, 65, 67, 117, 114])
EUTOPH = "".join(chr(c) for c in [70, 114, 101, 101, 80, 117, 109, 112, 75, 101, 121])
EXLSXU = "".join(chr(c) for c in [80, 117, 109, 112, 84, 105, 109, 101, 79, 117, 116])
FCRTFM = "".join(chr(c) for c in [69, 99, 111, 110, 83, 116, 97, 114, 116])
FEFJTA = 25
FEGZUQ = 104
FFTTID = 20
FJBIAM = 44
FJTACC = "".join(chr(c) for c in [67, 112, 79, 116, 79, 112, 116, 105, 111, 110])
FMNHTB = "".join(
    chr(c)
    for c in [
        69,
        99,
        111,
        110,
        80,
        114,
        111,
        103,
        65,
        118,
        97,
        105,
        108,
        97,
        98,
        108,
        101,
    ]
)
FTHECV = "".join(chr(c) for c in [80, 50, 72])
FTSIFJ = 56
FTTIDU = "".join(
    chr(c)
    for c in [
        70,
        97,
        110,
        67,
        108,
        114,
        70,
        111,
        114,
        99,
        101,
        79,
        102,
        102,
        84,
        101,
        109,
        112,
        65,
        100,
        99,
    ]
)
FXQGLR = "".join(chr(c) for c in [79, 117, 116, 72, 116, 114])
FYLJUI = "".join(
    chr(c)
    for c in [
        79,
        118,
        101,
        114,
        83,
        101,
        116,
        112,
        111,
        105,
        110,
        116,
        69,
        110,
        97,
        98,
        108,
        101,
    ]
)
GDSBDJ = 11
GQPLSP = "".join(chr(c) for c in [83, 116, 97, 110, 100, 97, 114, 100])
GSELHB = 78
GTYIYW = "".join(chr(c) for c in [80, 72])
GVUNXN = "".join(chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 54])
GYOUSP = "".join(chr(c) for c in [80, 49])
GZUQEX = "".join(chr(c) for c in [79, 117, 116, 73, 79, 52])
HBQNRX = 80
HBSKSO = "".join(chr(c) for c in [79, 117, 116, 51, 65, 67, 117, 114])
HBVWVU = 7
HECVYY = "".join(chr(c) for c in [80, 51, 72])
HEOCTH = 107
HFTHEC = "".join(chr(c) for c in [80, 49, 76])
HIUSOO = 111
HTBJEU = "".join(chr(c) for c in [85, 68, 80, 114, 111, 103, 69, 99, 111, 110])
HUGTYI = 67
HUOJRJ = "".join(chr(c) for c in [79, 117, 116, 55, 65])
HWDAFI = "".join(chr(c) for c in [50, 52, 104])
HXEKVK = 92
IACQFF = 18
IAMJMA = "".join(chr(c) for c in [83, 101, 116, 112, 111, 105, 110, 116, 71])
IBXTIA = "".join(
    chr(c)
    for c in [
        70,
        97,
        110,
        65,
        115,
        67,
        111,
        111,
        108,
        105,
        110,
        103,
        68,
        101,
        118,
        105,
        99,
        101,
    ]
)
IBXYBQ = 86
ICXQIE = "".join(chr(c) for c in [79, 117, 116, 50, 65])
IDNIBX = "".join(
    chr(c)
    for c in [
        70,
        97,
        110,
        70,
        111,
        114,
        99,
        101,
        79,
        70,
        70,
        79,
        112,
        116,
        105,
        111,
        110,
    ]
)
IDUBSS = 22
IEFXQG = "".join(chr(c) for c in [79, 117, 116, 51, 65])
IFJBIA = "".join(chr(c) for c in [83, 97, 110, 105, 76, 101, 118, 101, 108])
IGYOUS = 117
IIDNIB = "".join(
    chr(c)
    for c in [
        70,
        97,
        110,
        65,
        99,
        116,
        105,
        118,
        101,
        79,
        110,
        80,
        117,
        109,
        112,
        82,
        117,
        110,
    ]
)
IJUGSE = "".join(
    chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 51, 78, 98, 87, 101, 101, 107]
)
IKFWRK = "".join(chr(c) for c in [54, 48, 72, 90])
INEJNI = "".join(chr(c) for c in [50, 51, 48, 86, 65, 67])
IPIVLA = "".join(chr(c) for c in [79, 51])
IPOUYN = "".join(chr(c) for c in [80, 117, 109, 112, 85, 68])
IRYXBQ = "".join(chr(c) for c in [78, 111])
IUSOOQ = "".join(chr(c) for c in [79, 117, 116, 53, 65, 67, 117, 114])
IUXFEF = "".join(chr(c) for c in [65, 76, 87, 65, 89, 83, 95, 79, 78])
IVDNQG = 69
IVLASS = "".join(chr(c) for c in [])
IYWSKW = "".join(chr(c) for c in [67, 108, 101, 97, 110, 67, 111, 118, 101, 114])
JBIAMJ = "".join(chr(c) for c in [83, 97, 110, 105, 79, 110, 84, 105, 109, 101])
JEUTOP = "".join(chr(c) for c in [76, 97, 115, 116, 80, 117, 109, 112, 75, 101, 121])
JHIUSO = "".join(chr(c) for c in [79, 117, 116, 52, 65, 67, 117, 114])
JIGYOU = "".join(chr(c) for c in [68, 114, 97, 105, 110, 77, 111, 100, 101])
JJJVYF = 36
JJVYFC = "".join(chr(c) for c in [79, 110, 122, 101, 110, 68, 117, 114])
JMAOAW = 13
JMCBFE = 103
JNIBXY = 85
JPUNRJ = 8
JQRJJJ = "".join(chr(c) for c in [70, 105, 108, 116, 68, 117, 114])
JRJHIU = 116
JTACCP = 23
JUGSEL = 77
JUIKFW = 62
JUTYEK = "".join(chr(c) for c in [83, 104, 97, 114, 101, 100])
JVDQLA = "".join(
    chr(c)
    for c in [
        81,
        117,
        105,
        101,
        116,
        65,
        99,
        116,
        105,
        111,
        110,
        79,
        110,
        72,
        101,
        97,
        116,
        101,
        114,
    ]
)
JVHFTH = "".join(chr(c) for c in [78, 65])
JVYFCR = 38
JWMNZM = "".join(
    chr(c)
    for c in [
        67,
        112,
        79,
        102,
        102,
        84,
        105,
        109,
        101,
        68,
        117,
        114,
        105,
        110,
        103,
        79,
        84,
    ]
)
JYKLGQ = 54
JYMOUN = 0
KCWAON = 61
KFWRKI = "".join(chr(c) for c in [53, 48, 72, 90])
KINEJN = "".join(chr(c) for c in [50, 52, 48, 86, 97, 99])
KLGQPL = "".join(chr(c) for c in [79, 51, 84, 121, 112, 101])
KMLOIJ = "".join(
    chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 49, 78, 98, 87, 101, 101, 107]
)
KPHUOJ = "".join(chr(c) for c in [79, 117, 116, 54, 65])
KQXPIC = "".join(chr(c) for c in [86, 65, 76, 86, 69])
KSOKPH = 98
KVKZIL = "".join(
    chr(c)
    for c in [85, 68, 65, 99, 116, 105, 111, 110, 79, 110, 81, 117, 105, 101, 116]
)
KWIVDN = 68
KXSJWM = 50
KZILXW = "".join(
    chr(c) for c in [82, 101, 115, 116, 111, 114, 101, 65, 108, 108, 85, 68]
)
LASSAK = "".join(chr(c) for c in [70, 117, 108, 108, 79, 110])
LGQPLS = 59
LHBQNR = "".join(
    chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 54, 78, 98, 87, 101, 101, 107]
)
LIUXFE = "".join(chr(c) for c in [83, 84, 65, 78, 68, 65, 82, 68])
LJUIKF = "".join(
    chr(c)
    for c in [
        69,
        99,
        111,
        110,
        66,
        101,
        108,
        111,
        119,
        83,
        101,
        116,
        112,
        111,
        105,
        110,
        116,
    ]
)
LKXSJW = "".join(
    chr(c)
    for c in [70, 105, 108, 116, 67, 80, 79, 84, 68, 101, 97, 100, 98, 97, 110, 100]
)
LNMHXE = "".join(chr(c) for c in [66, 114, 101, 97, 107, 101, 114, 83, 101, 116, 52])
LOIJUG = "".join(
    chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 50, 78, 98, 87, 101, 101, 107]
)
LRAHEO = "".join(chr(c) for c in [79, 117, 116, 49, 65, 67, 117, 114])
LSPFTS = "".join(
    chr(c) for c in [79, 51, 83, 117, 115, 112, 101, 110, 100, 84, 105, 109, 101]
)
LSXUJU = "".join(
    chr(c) for c in [76, 105, 103, 104, 116, 84, 105, 109, 101, 79, 117, 116]
)
LXWAJV = "".join(
    chr(c)
    for c in [81, 117, 105, 101, 116, 65, 99, 116, 105, 111, 110, 79, 110, 85, 68]
)
MAOAWB = "".join(chr(c) for c in [67])
MCBFEG = "".join(chr(c) for c in [76, 73])
MHXEKV = "".join(chr(c) for c in [66, 114, 101, 97, 107, 101, 114, 83, 101, 116, 53])
MJIGYO = 53
MJMAOA = "".join(chr(c) for c in [84, 101, 109, 112, 85, 110, 105, 116, 115])
MJVHFT = 93
MLOIJU = 75
MNHTBJ = "".join(chr(c) for c in [65, 118, 97, 105, 108, 97, 98, 108, 101])
MNZMJI = "".join(
    chr(c)
    for c in [
        70,
        105,
        108,
        116,
        79,
        110,
        84,
        105,
        109,
        101,
        68,
        117,
        114,
        105,
        110,
        103,
        79,
        84,
    ]
)
MOUNBL = 46
NBLKXS = "".join(
    chr(c) for c in [70, 105, 108, 116, 67, 80, 79, 84, 65, 99, 116, 84, 101, 109, 112]
)
NIBXTI = "".join(chr(c) for c in [70, 97, 110, 80, 117, 109, 112])
NIBXYB = "".join(
    chr(c) for c in [66, 114, 78, 98, 83, 101, 116, 116, 105, 110, 103, 115]
)
NKMLOI = 74
NMHXEK = 91
NPYYLI = "".join(chr(c) for c in [70, 105, 108, 116, 80, 49])
NQGVUN = "".join(chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 53])
NQJYMO = "".join(chr(c) for c in [72, 105])
NQLNMH = "".join(chr(c) for c in [66, 114, 101, 97, 107, 101, 114, 83, 101, 116, 51])
NRSJMC = "".join(chr(c) for c in [68, 105, 114, 101, 99, 116, 67, 117, 114])
NRXCHW = "".join(
    chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 56, 78, 98, 87, 101, 101, 107]
)
NXNKML = 73
NZMJIG = 48
OCTHBS = 108
OIJUGS = 76
OJRJHI = "".join(chr(c) for c in [68, 105, 114, 101, 99, 116])
OKPHUO = 99
ONPYYL = "".join(chr(c) for c in [70, 105, 108, 116, 67, 80])
OOQNRS = 113
OPHUGT = "".join(chr(c) for c in [75, 54, 48, 48, 70, 111, 114, 77, 97, 121])
OQNRSJ = "".join(chr(c) for c in [79, 117, 116, 55, 65, 67, 117, 114])
OUNBLK = "".join(
    chr(c) for c in [70, 105, 108, 116, 79, 84, 68, 101, 97, 100, 98, 97, 110, 100]
)
OUSPBW = "".join(chr(c) for c in [79, 51, 85, 115, 97, 103, 101])
OUYNQJ = 6
PBWJYK = "".join(chr(c) for c in [65, 108, 119, 97, 121, 115])
PFTSIF = "".join(chr(c) for c in [79, 51, 83, 117, 115, 112, 101, 110, 100, 66, 121])
PHUGTY = "".join(chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 49])
PHUOJR = 100
PICXQI = 94
PIPIVL = "".join(chr(c) for c in [67, 80])
PIVLAS = "".join(chr(c) for c in [76, 49, 50, 48])
PQIPOU = "".join(
    chr(c) for c in [70, 105, 108, 116, 83, 117, 115, 112, 101, 110, 100, 66, 121]
)
QBMJVH = "".join(chr(c) for c in [65, 76, 76])
QEXLSX = 84
QFFTTI = "".join(
    chr(c)
    for c in [
        70,
        97,
        110,
        83,
        101,
        116,
        70,
        111,
        114,
        99,
        101,
        79,
        102,
        102,
        84,
        101,
        109,
        112,
        65,
        100,
        99,
    ]
)
QFYLJU = 14
QGLRAH = "".join(chr(c) for c in [72, 84, 82])
QGVUNX = 71
QIEFXQ = 96
QIPOUY = "".join(chr(c) for c in [65, 110, 121, 85, 68])
QLAIID = "".join(chr(c) for c in [75, 101, 101, 112, 79, 102, 102])
QLNMHX = 90
QNRSJM = 114
QNRXCH = 81
QPLSPF = "".join(chr(c) for c in [84, 111, 103, 103, 108, 101])
QRJJJV = 30
QSNQLN = "".join(chr(c) for c in [66, 114, 101, 97, 107, 101, 114, 83, 101, 116, 50])
RAHEOC = 106
RJJJVY = "".join(chr(c) for c in [79, 110, 122, 101, 110, 83, 116, 97, 114, 116])
RKINEJ = "".join(chr(c) for c in [76, 105, 110, 101, 86, 111, 108, 116, 97, 103, 101])
RSJMCB = 115
RTFMNH = "".join(chr(c) for c in [69, 99, 111, 110, 83, 116, 111, 112])
RXCHWD = 82
RYXBQF = "".join(
    chr(c) for c in [70, 117, 108, 108, 80, 111, 119, 101, 114, 79, 110, 108, 121]
)
SAKQXP = "".join(chr(c) for c in [83, 65, 78, 73])
SBDJQR = 27
SELHBQ = "".join(
    chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 53, 78, 98, 87, 101, 101, 107]
)
SIFJBI = 57
SIRYXB = 83
SJMCBF = "".join(chr(c) for c in [79, 117, 116, 76, 105])
SJWMNZ = 51
SKSOKP = "".join(chr(c) for c in [79, 117, 116, 52, 65])
SKWIVD = "".join(chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 50])
SNQLNM = 89
SOKPHU = "".join(chr(c) for c in [79, 117, 116, 53, 65])
SOOQNR = "".join(chr(c) for c in [79, 117, 116, 54, 65, 67, 117, 114])
SPBWJY = "".join(chr(c) for c in [70, 105, 108, 116, 101, 114])
SPFTSI = 58
SSAKQX = "".join(chr(c) for c in [83, 112, 76, 105, 102, 116])
SSUHBV = "".join(
    chr(c)
    for c in [86, 83, 80, 67, 104, 101, 99, 107, 102, 108, 111, 83, 112, 101, 101, 100]
)
SUHBVW = 6
SXUJUT = 2
TACCPQ = "".join(chr(c) for c in [68, 105, 115, 97, 98, 108, 101, 100])
TBJEUT = "".join(
    chr(c)
    for c in [
        69,
        99,
        111,
        110,
        67,
        111,
        110,
        116,
        114,
        111,
        108,
        97,
        98,
        108,
        101,
        77,
        97,
        110,
        117,
        97,
        108,
        108,
        121,
    ]
)
TFMNHT = 65
THBSKS = 109
THECVY = "".join(chr(c) for c in [80, 50, 76])
TIACQF = "".join(
    chr(c)
    for c in [70, 97, 110, 67, 108, 114, 65, 79, 84, 84, 101, 109, 112, 65, 100, 99]
)
TIDUBS = "".join(
    chr(c)
    for c in [
        70,
        97,
        110,
        70,
        111,
        114,
        99,
        101,
        79,
        102,
        102,
        73,
        110,
        104,
        105,
        98,
        105,
        116,
        68,
        101,
        108,
        97,
        121,
    ]
)
TOPHUG = "".join(
    chr(c)
    for c in [
        81,
        117,
        105,
        99,
        107,
        79,
        110,
        79,
        102,
        102,
        75,
        101,
        121,
        69,
        110,
        97,
        98,
        108,
        101,
    ]
)
TSIFJB = "".join(
    chr(c) for c in [79, 51, 70, 105, 108, 116, 101, 114, 68, 101, 108, 97, 121]
)
TTIDUB = 21
TYIYWS = "".join(chr(c) for c in [67, 104, 101, 99, 107, 71, 70, 67, 73])
UBSSUH = 5
UBYGDS = "".join(
    chr(c) for c in [86, 83, 80, 83, 112, 101, 101, 100, 76, 101, 118, 101, 108, 50]
)
UGSELH = "".join(
    chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 52, 78, 98, 87, 101, 101, 107]
)
UGTYIY = "".join(chr(c) for c in [67, 108, 101, 97, 110, 70, 105, 108, 116, 101, 114])
UHBVWV = "".join(
    chr(c) for c in [86, 83, 80, 70, 105, 108, 116, 101, 114, 83, 112, 101, 101, 100]
)
UIKFWR = "".join(
    chr(c) for c in [76, 105, 110, 101, 70, 114, 101, 113, 117, 101, 110, 99, 121]
)
UJUTYE = 1
UNBLKX = 47
UNXNKM = "".join(chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 55])
UOJRJH = 101
UQEXLS = "".join(chr(c) for c in [76, 105, 103, 104, 116, 73, 110, 116, 115])
USOOQN = 112
USPBWJ = 55
UTYEKC = "".join(chr(c) for c in [79, 119, 110])
UYNQJY = "".join(chr(c) for c in [80, 117, 114, 103, 101, 83, 112, 101, 101, 100])
VDNQGV = "".join(chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 52])
VDQLAI = "".join(chr(c) for c in [78, 111, 65, 99, 116, 105, 111, 110])
VHFTHE = "".join(chr(c) for c in [80, 49, 72])
VKZILX = "".join(chr(c) for c in [73, 103, 110, 111, 114, 101, 85, 68])
VLASSA = "".join(chr(c) for c in [70, 65, 78])
VUBYGD = 9
VUNXNK = 72
VWVUBY = 8
VYFCRT = "".join(chr(c) for c in [79, 110, 122, 101, 110, 70, 114, 101, 113])
VYYPIP = "".join(chr(c) for c in [80, 52, 76])
WAJVDQ = "".join(chr(c) for c in [80, 97, 117, 115, 101])
WAONPY = 26
WBSIRY = 12
WIVDNQ = "".join(chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 51])
WJYKLG = "".join(chr(c) for c in [79, 51, 80, 117, 109, 112])
WMNZMJ = 52
WRKINE = 1
WVUBYG = "".join(
    chr(c) for c in [86, 83, 80, 83, 112, 101, 101, 100, 76, 101, 118, 101, 108, 49]
)
XBQFYL = 3
XCHWDA = "".join(chr(c) for c in [84, 105, 109, 101, 70, 111, 114, 109, 97, 116])
XEKVKZ = "".join(
    chr(c) for c in [67, 69, 95, 50, 120, 50, 48, 65, 108, 108, 111, 119, 101, 100]
)
XFEFJT = "".join(
    chr(c) for c in [70, 105, 108, 116, 79, 84, 79, 112, 116, 105, 111, 110]
)
XLSXUJ = 4
XNKMLO = "".join(chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 56])
XPICXQ = "".join(chr(c) for c in [79, 117, 116, 49, 66])
XQGLRA = 102
XQIEFX = "".join(chr(c) for c in [79, 117, 116, 50, 66])
XSJWMN = "".join(
    chr(c)
    for c in [67, 112, 79, 110, 84, 105, 109, 101, 68, 117, 114, 105, 110, 103, 79, 84]
)
XTIACQ = 17
XUJUTY = "".join(chr(c) for c in [76, 49, 50, 48, 84, 105, 109, 101, 114])
XWAJVD = "".join(chr(c) for c in [67, 97, 110, 99, 101, 108])
XYBQSN = 87
YBQSNQ = "".join(chr(c) for c in [66, 114, 101, 97, 107, 101, 114, 83, 101, 116, 49])
YEKCWA = 2
YFCRTF = 40
YGDSBD = "".join(
    chr(c) for c in [86, 83, 80, 83, 112, 101, 101, 100, 76, 101, 118, 101, 108, 51]
)
YIYWSK = "".join(chr(c) for c in [67, 104, 97, 110, 103, 101, 87, 97, 116, 101, 114])
YLIUXF = 24
YLJUIK = 5
YMOUNB = "".join(
    chr(c) for c in [70, 105, 108, 116, 79, 84, 65, 99, 116, 84, 101, 109, 112]
)
YNQJYM = "".join(chr(c) for c in [76, 111])
YPIPIV = "".join(chr(c) for c in [66, 76, 79])
YWSKWI = "".join(
    chr(c) for c in [82, 101, 112, 108, 97, 99, 101, 70, 105, 108, 116, 101, 114]
)
YYLIUX = "".join(chr(c) for c in [67, 112, 85, 115, 97, 103, 101])
YYPIPI = "".join(chr(c) for c in [80, 53])
ZCQBMJ = "".join(
    chr(c) for c in [67, 111, 110, 102, 105, 103, 78, 117, 109, 98, 101, 114]
)
ZILXWA = "".join(
    chr(c) for c in [67, 97, 110, 99, 101, 108, 79, 116, 104, 101, 114, 85, 68]
)
ZMJIGY = "".join(
    chr(c)
    for c in [70, 105, 108, 116, 83, 117, 115, 112, 101, 110, 100, 84, 105, 109, 101]
)
ZUQEXL = 105
AJVDQL = [XWAJVD, WAJVDQ]
BWJYKL = [SPBWJY, PBWJYK]
CBFEGZ = [
    JVHFTH,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    MCBFEG,
]
CPQIPO = [TACCPQ, ACCPQI, CCPQIP]
EGZUQE = [
    JVHFTH,
    VHFTHE,
    HFTHEC,
    FTHECV,
    THECVY,
    HECVYY,
    ECVYYP,
    CVYYPI,
    VYYPIP,
    YYPIPI,
    YPIPIV,
    PIPIVL,
    IPIVLA,
    PIVLAS,
    IVLASS,
    MCBFEG,
    VLASSA,
    IVLASS,
    IVLASS,
    LASSAK,
    ASSAKQ,
    SSAKQX,
    SAKQXP,
    AKQXPI,
    KQXPIC,
]
FIKJPU = [
    BMJVHF,
    XPICXQ,
    ICXQIE,
    XQIEFX,
    IEFXQG,
    FXQGLR,
    SKSOKP,
    SOKPHU,
    KPHUOJ,
    HUOJRJ,
    OJRJHI,
    SJMCBF,
    BFEGZU,
    GZUQEX,
]
FWRKIN = [IKFWRK, KFWRKI]
GLRAHE = [
    JVHFTH,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    QGLRAH,
]
IKJPUN = [MCBFEG]
ILXWAJ = [VKZILX, KZILXW, JVHFTH, ZILXWA]
KJPUNR = []
LAIIDN = [VDQLAI, DQLAII, QLAIID, IVLASS]
NEJNIB = [KINEJN, INEJNI]
NHTBJE = [JVHFTH, MNHTBJ]
OAWBSI = [MAOAWB, AOAWBS]
PLSPFT = [GQPLSP, QPLSPF]
POUYNQ = [QIPOUY, IPOUYN]
PYYLIU = [AONPYY, ONPYYL, NPYYLI]
QJYMOU = [YNQJYM, NQJYMO]
QXPICX = [
    JVHFTH,
    VHFTHE,
    HFTHEC,
    FTHECV,
    THECVY,
    HECVYY,
    ECVYYP,
    CVYYPI,
    VYYPIP,
    YYPIPI,
    YPIPIV,
    PIPIVL,
    IPIVLA,
    PIVLAS,
    IVLASS,
    IVLASS,
    VLASSA,
    IVLASS,
    IVLASS,
    LASSAK,
    ASSAKQ,
    SSAKQX,
    SAKQXP,
    AKQXPI,
    KQXPIC,
]
RJHIUS = [
    JVHFTH,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    PIPIVL,
]
TYEKCW = [JUTYEK, UTYEKC]
UTOPHU = [JEUTOP, EUTOPH]
UXFEFJ = [LIUXFE, IUXFEF]
WDAFIK = [CHWDAF, HWDAFI]
WSKWIV = [UGTYIY, GTYIYW, TYIYWS, YIYWSK, IYWSKW, YWSKWI]
YKLGQP = [PIPIVL, GYOUSP]
YOUSPB = [JVHFTH, GYOUSP, PIPIVL]
YXBQFY = [IRYXBQ, RYXBQF]


class GeckoConfigStruct:
    def __init__(self, struct_):
        self.struct = struct_

    @property
    def version(self):
        return JPUNRJ

    @property
    def output_keys(self):
        return FIKJPU

    @property
    def accessors(self):
        return {
            ZCQBMJ: GeckoByteStructAccessor(self.struct, ZCQBMJ, CQBMJV, QBMJVH),
            BMJVHF: GeckoEnumStructAccessor(
                self.struct, BMJVHF, MJVHFT, None, QXPICX, None, None, QBMJVH
            ),
            XPICXQ: GeckoEnumStructAccessor(
                self.struct, XPICXQ, PICXQI, None, QXPICX, None, None, QBMJVH
            ),
            ICXQIE: GeckoEnumStructAccessor(
                self.struct, ICXQIE, CXQIEF, None, QXPICX, None, None, QBMJVH
            ),
            XQIEFX: GeckoEnumStructAccessor(
                self.struct, XQIEFX, QIEFXQ, None, QXPICX, None, None, QBMJVH
            ),
            IEFXQG: GeckoEnumStructAccessor(
                self.struct, IEFXQG, EFXQGL, None, QXPICX, None, None, QBMJVH
            ),
            FXQGLR: GeckoEnumStructAccessor(
                self.struct, FXQGLR, XQGLRA, None, GLRAHE, None, None, QBMJVH
            ),
            LRAHEO: GeckoByteStructAccessor(self.struct, LRAHEO, RAHEOC, QBMJVH),
            AHEOCT: GeckoByteStructAccessor(self.struct, AHEOCT, HEOCTH, QBMJVH),
            EOCTHB: GeckoByteStructAccessor(self.struct, EOCTHB, OCTHBS, QBMJVH),
            CTHBSK: GeckoByteStructAccessor(self.struct, CTHBSK, THBSKS, QBMJVH),
            HBSKSO: GeckoByteStructAccessor(self.struct, HBSKSO, BSKSOK, QBMJVH),
            SKSOKP: GeckoEnumStructAccessor(
                self.struct, SKSOKP, KSOKPH, None, QXPICX, None, None, QBMJVH
            ),
            SOKPHU: GeckoEnumStructAccessor(
                self.struct, SOKPHU, OKPHUO, None, QXPICX, None, None, QBMJVH
            ),
            KPHUOJ: GeckoEnumStructAccessor(
                self.struct, KPHUOJ, PHUOJR, None, QXPICX, None, None, QBMJVH
            ),
            HUOJRJ: GeckoEnumStructAccessor(
                self.struct, HUOJRJ, UOJRJH, None, QXPICX, None, None, QBMJVH
            ),
            OJRJHI: GeckoEnumStructAccessor(
                self.struct, OJRJHI, JRJHIU, None, RJHIUS, None, None, QBMJVH
            ),
            JHIUSO: GeckoByteStructAccessor(self.struct, JHIUSO, HIUSOO, QBMJVH),
            IUSOOQ: GeckoByteStructAccessor(self.struct, IUSOOQ, USOOQN, QBMJVH),
            SOOQNR: GeckoByteStructAccessor(self.struct, SOOQNR, OOQNRS, QBMJVH),
            OQNRSJ: GeckoByteStructAccessor(self.struct, OQNRSJ, QNRSJM, QBMJVH),
            NRSJMC: GeckoByteStructAccessor(self.struct, NRSJMC, RSJMCB, QBMJVH),
            SJMCBF: GeckoEnumStructAccessor(
                self.struct, SJMCBF, JMCBFE, None, CBFEGZ, None, None, QBMJVH
            ),
            BFEGZU: GeckoEnumStructAccessor(
                self.struct, BFEGZU, FEGZUQ, None, EGZUQE, None, None, QBMJVH
            ),
            GZUQEX: GeckoEnumStructAccessor(
                self.struct, GZUQEX, ZUQEXL, None, EGZUQE, None, None, QBMJVH
            ),
            UQEXLS: GeckoByteStructAccessor(self.struct, UQEXLS, QEXLSX, QBMJVH),
            EXLSXU: GeckoByteStructAccessor(self.struct, EXLSXU, XLSXUJ, QBMJVH),
            LSXUJU: GeckoByteStructAccessor(self.struct, LSXUJU, SXUJUT, QBMJVH),
            XUJUTY: GeckoEnumStructAccessor(
                self.struct, XUJUTY, UJUTYE, YEKCWA, TYEKCW, None, YEKCWA, QBMJVH
            ),
            EKCWAO: GeckoByteStructAccessor(self.struct, EKCWAO, KCWAON, QBMJVH),
            CWAONP: GeckoEnumStructAccessor(
                self.struct, CWAONP, WAONPY, None, PYYLIU, None, None, QBMJVH
            ),
            YYLIUX: GeckoEnumStructAccessor(
                self.struct, YYLIUX, YLIUXF, None, UXFEFJ, None, None, QBMJVH
            ),
            XFEFJT: GeckoBoolStructAccessor(
                self.struct, XFEFJT, FEFJTA, EFJTAC, QBMJVH
            ),
            FJTACC: GeckoEnumStructAccessor(
                self.struct, FJTACC, JTACCP, None, CPQIPO, None, None, QBMJVH
            ),
            PQIPOU: GeckoEnumStructAccessor(
                self.struct, PQIPOU, FEFJTA, OUYNQJ, POUYNQ, None, YEKCWA, QBMJVH
            ),
            UYNQJY: GeckoEnumStructAccessor(
                self.struct, UYNQJY, FEFJTA, JYMOUN, QJYMOU, None, YEKCWA, QBMJVH
            ),
            YMOUNB: GeckoByteStructAccessor(self.struct, YMOUNB, MOUNBL, QBMJVH),
            OUNBLK: GeckoByteStructAccessor(self.struct, OUNBLK, UNBLKX, QBMJVH),
            NBLKXS: GeckoByteStructAccessor(self.struct, NBLKXS, BLKXSJ, QBMJVH),
            LKXSJW: GeckoByteStructAccessor(self.struct, LKXSJW, KXSJWM, QBMJVH),
            XSJWMN: GeckoByteStructAccessor(self.struct, XSJWMN, SJWMNZ, QBMJVH),
            JWMNZM: GeckoByteStructAccessor(self.struct, JWMNZM, WMNZMJ, QBMJVH),
            MNZMJI: GeckoByteStructAccessor(self.struct, MNZMJI, NZMJIG, QBMJVH),
            ZMJIGY: GeckoByteStructAccessor(self.struct, ZMJIGY, MJIGYO, QBMJVH),
            JIGYOU: GeckoEnumStructAccessor(
                self.struct, JIGYOU, IGYOUS, None, YOUSPB, None, None, QBMJVH
            ),
            OUSPBW: GeckoEnumStructAccessor(
                self.struct, OUSPBW, USPBWJ, None, BWJYKL, None, None, QBMJVH
            ),
            WJYKLG: GeckoEnumStructAccessor(
                self.struct, WJYKLG, JYKLGQ, None, YKLGQP, None, None, QBMJVH
            ),
            KLGQPL: GeckoEnumStructAccessor(
                self.struct, KLGQPL, LGQPLS, None, PLSPFT, None, None, QBMJVH
            ),
            LSPFTS: GeckoByteStructAccessor(self.struct, LSPFTS, SPFTSI, QBMJVH),
            PFTSIF: GeckoEnumStructAccessor(
                self.struct, PFTSIF, FTSIFJ, None, POUYNQ, None, None, QBMJVH
            ),
            TSIFJB: GeckoByteStructAccessor(self.struct, TSIFJB, SIFJBI, QBMJVH),
            IFJBIA: GeckoByteStructAccessor(self.struct, IFJBIA, FJBIAM, QBMJVH),
            JBIAMJ: GeckoByteStructAccessor(self.struct, JBIAMJ, BIAMJM, QBMJVH),
            IAMJMA: GeckoTempStructAccessor(self.struct, IAMJMA, AMJMAO, QBMJVH),
            MJMAOA: GeckoEnumStructAccessor(
                self.struct, MJMAOA, JMAOAW, YEKCWA, OAWBSI, None, YEKCWA, QBMJVH
            ),
            AWBSIR: GeckoEnumStructAccessor(
                self.struct, AWBSIR, WBSIRY, None, YKLGQP, None, None, QBMJVH
            ),
            BSIRYX: GeckoEnumStructAccessor(
                self.struct, BSIRYX, SIRYXB, XBQFYL, YXBQFY, None, YEKCWA, QBMJVH
            ),
            BQFYLJ: GeckoBoolStructAccessor(
                self.struct, BQFYLJ, QFYLJU, EFJTAC, QBMJVH
            ),
            FYLJUI: GeckoBoolStructAccessor(
                self.struct, FYLJUI, QFYLJU, YLJUIK, QBMJVH
            ),
            LJUIKF: GeckoByteStructAccessor(self.struct, LJUIKF, JUIKFW, QBMJVH),
            UIKFWR: GeckoEnumStructAccessor(
                self.struct, UIKFWR, SIRYXB, WRKINE, FWRKIN, None, YEKCWA, QBMJVH
            ),
            RKINEJ: GeckoEnumStructAccessor(
                self.struct, RKINEJ, SIRYXB, YEKCWA, NEJNIB, None, YEKCWA, QBMJVH
            ),
            EJNIBX: GeckoByteStructAccessor(self.struct, EJNIBX, JNIBXY, None),
            NIBXYB: GeckoByteStructAccessor(self.struct, NIBXYB, IBXYBQ, None),
            BXYBQS: GeckoByteStructAccessor(self.struct, BXYBQS, XYBQSN, None),
            YBQSNQ: GeckoByteStructAccessor(self.struct, YBQSNQ, BQSNQL, QBMJVH),
            QSNQLN: GeckoByteStructAccessor(self.struct, QSNQLN, SNQLNM, QBMJVH),
            NQLNMH: GeckoByteStructAccessor(self.struct, NQLNMH, QLNMHX, QBMJVH),
            LNMHXE: GeckoByteStructAccessor(self.struct, LNMHXE, NMHXEK, QBMJVH),
            MHXEKV: GeckoByteStructAccessor(self.struct, MHXEKV, HXEKVK, QBMJVH),
            XEKVKZ: GeckoBoolStructAccessor(
                self.struct, XEKVKZ, EKVKZI, OUYNQJ, QBMJVH
            ),
            KVKZIL: GeckoEnumStructAccessor(
                self.struct, KVKZIL, EKVKZI, WRKINE, ILXWAJ, None, EFJTAC, QBMJVH
            ),
            LXWAJV: GeckoEnumStructAccessor(
                self.struct, LXWAJV, EKVKZI, JYMOUN, AJVDQL, None, YEKCWA, QBMJVH
            ),
            JVDQLA: GeckoEnumStructAccessor(
                self.struct, JVDQLA, JMAOAW, XBQFYL, LAIIDN, None, EFJTAC, QBMJVH
            ),
            AIIDNI: GeckoBoolStructAccessor(
                self.struct, AIIDNI, JMAOAW, YLJUIK, QBMJVH
            ),
            IIDNIB: GeckoBoolStructAccessor(
                self.struct, IIDNIB, JMAOAW, OUYNQJ, QBMJVH
            ),
            IDNIBX: GeckoBoolStructAccessor(
                self.struct, IDNIBX, JMAOAW, DNIBXT, QBMJVH
            ),
            NIBXTI: GeckoEnumStructAccessor(
                self.struct, NIBXTI, QFYLJU, WRKINE, YOUSPB, None, EFJTAC, QBMJVH
            ),
            IBXTIA: GeckoBoolStructAccessor(
                self.struct, IBXTIA, QFYLJU, XBQFYL, QBMJVH
            ),
            BXTIAC: GeckoByteStructAccessor(self.struct, BXTIAC, XTIACQ, QBMJVH),
            TIACQF: GeckoByteStructAccessor(self.struct, TIACQF, IACQFF, QBMJVH),
            ACQFFT: GeckoByteStructAccessor(self.struct, ACQFFT, CQFFTT, QBMJVH),
            QFFTTI: GeckoByteStructAccessor(self.struct, QFFTTI, FFTTID, QBMJVH),
            FTTIDU: GeckoByteStructAccessor(self.struct, FTTIDU, TTIDUB, QBMJVH),
            TIDUBS: GeckoByteStructAccessor(self.struct, TIDUBS, IDUBSS, QBMJVH),
            DUBSSU: GeckoBoolStructAccessor(
                self.struct, DUBSSU, UBSSUH, JYMOUN, QBMJVH
            ),
            BSSUHB: GeckoBoolStructAccessor(
                self.struct, BSSUHB, UBSSUH, YEKCWA, QBMJVH
            ),
            SSUHBV: GeckoByteStructAccessor(self.struct, SSUHBV, SUHBVW, QBMJVH),
            UHBVWV: GeckoByteStructAccessor(self.struct, UHBVWV, HBVWVU, QBMJVH),
            BVWVUB: GeckoByteStructAccessor(self.struct, BVWVUB, VWVUBY, QBMJVH),
            WVUBYG: GeckoByteStructAccessor(self.struct, WVUBYG, VUBYGD, QBMJVH),
            UBYGDS: GeckoByteStructAccessor(self.struct, UBYGDS, BYGDSB, QBMJVH),
            YGDSBD: GeckoByteStructAccessor(self.struct, YGDSBD, GDSBDJ, QBMJVH),
            DSBDJQ: GeckoByteStructAccessor(self.struct, DSBDJQ, SBDJQR, QBMJVH),
            BDJQRJ: GeckoTimeStructAccessor(self.struct, BDJQRJ, DJQRJJ, QBMJVH),
            JQRJJJ: GeckoTimeStructAccessor(self.struct, JQRJJJ, QRJJJV, QBMJVH),
            RJJJVY: GeckoTimeStructAccessor(self.struct, RJJJVY, JJJVYF, QBMJVH),
            JJVYFC: GeckoTimeStructAccessor(self.struct, JJVYFC, JVYFCR, QBMJVH),
            VYFCRT: GeckoByteStructAccessor(self.struct, VYFCRT, YFCRTF, QBMJVH),
            FCRTFM: GeckoTimeStructAccessor(self.struct, FCRTFM, CRTFMN, QBMJVH),
            RTFMNH: GeckoTimeStructAccessor(self.struct, RTFMNH, TFMNHT, QBMJVH),
            FMNHTB: GeckoEnumStructAccessor(
                self.struct, FMNHTB, EKVKZI, EFJTAC, NHTBJE, None, YEKCWA, QBMJVH
            ),
            HTBJEU: GeckoBoolStructAccessor(
                self.struct, HTBJEU, EKVKZI, YLJUIK, QBMJVH
            ),
            TBJEUT: GeckoBoolStructAccessor(
                self.struct, TBJEUT, EKVKZI, XBQFYL, QBMJVH
            ),
            BJEUTO: GeckoEnumStructAccessor(
                self.struct, BJEUTO, UJUTYE, JYMOUN, UTOPHU, None, YEKCWA, QBMJVH
            ),
            TOPHUG: GeckoBoolStructAccessor(
                self.struct, TOPHUG, UJUTYE, WRKINE, QBMJVH
            ),
            XCHWDA: GeckoEnumStructAccessor(
                self.struct, XCHWDA, SIRYXB, JYMOUN, WDAFIK, None, YEKCWA, QBMJVH
            ),
            DAFIKJ: GeckoBoolStructAccessor(
                self.struct, DAFIKJ, QFYLJU, JYMOUN, QBMJVH
            ),
            AFIKJP: GeckoBoolStructAccessor(
                self.struct, AFIKJP, FEFJTA, WRKINE, QBMJVH
            ),
        }



""" GeckoAsyncSpa class """

from datetime import datetime, timezone
import logging
import asyncio
import importlib
import socket
import time
from typing import Optional

from .async_spa_descriptor import GeckoAsyncSpaDescriptor
from .async_tasks import AsyncTasks
from .spa_events import GeckoSpaEvent

from .const import GeckoConstants
from .driver import (
    GeckoAsyncUdpProtocol,
    GeckoPacketProtocolHandler,
    GeckoPingProtocolHandler,
    GeckoVersionProtocolHandler,
    GeckoGetChannelProtocolHandler,
    GeckoConfigFileProtocolHandler,
    GeckoStatusBlockProtocolHandler,
    GeckoAsyncPartialStatusBlockProtocolHandler,
    GeckoPackCommandProtocolHandler,
    GeckoRFErrProtocolHandler,
    GeckoUnhandledProtocolHandler,
    GeckoWatercareProtocolHandler,
    GeckoWatercareErrorHandler,
    # Rest
    GeckoAsyncStructure,
    Observable,
)

_LOGGER = logging.getLogger(__name__)


class GeckoAsyncSpa(Observable):
    """GeckoAsyncSpa class manages an instance of a spa, and is the main point of
    contact for control and monitoring. Uses the declarations found in pack/* to build
    an object that exposes the properties and capabilities of your spa. This class
    should only be used via a Facade which hides the implementation details"""

    def __init__(
        self,
        client_id: bytes,
        spa_descriptor: GeckoAsyncSpaDescriptor,
        taskman: AsyncTasks,
        event_handler: GeckoSpaEvent.CallBack,
    ) -> None:
        super().__init__()

        self.client_id = client_id
        self.descriptor = spa_descriptor
        self._taskman = taskman
        self._event_handler: GeckoSpaEvent.CallBack = event_handler

        self._con_lost: Optional[asyncio.Future] = None
        self._transport: Optional[asyncio.BaseTransport] = None
        self._protocol: Optional[GeckoAsyncUdpProtocol] = None
        self._is_connected = False

        self.intouch_version_en = ""
        self.intouch_version_co = ""

        self.channel = 0
        self.signal = 0

        self.config_version = 0
        self.log_version = 0

        self.pack_class = None
        self.pack_type = ""
        self.config_class = None
        self.log_class = None

        self.pack = None
        self.version = ""
        self.config_number = 0

        self.struct = GeckoAsyncStructure(self._on_set_value, self._on_async_set_value)
        self._last_ping_at: Optional[datetime] = None

    @property
    def sendparms(self) -> tuple:
        return (
            self.descriptor.destination[0],
            self.descriptor.destination[1],
            self.descriptor.identifier,
            self.client_id,
        )

    @property
    def is_connected(self) -> bool:
        return self._is_connected

    @property
    def revision(self) -> str:
        """Get the revision of the spa pack structure used to generate
        the pack modules"""
        # TODO: Need to add base classes and type hinting to auto-generated
        # pack code and remove type-hint suppression below
        return self.pack_class.revision  # type: ignore

    def _get_version_handler_func(self) -> GeckoVersionProtocolHandler:
        assert self._protocol is not None
        return GeckoVersionProtocolHandler.request(
            self._protocol.get_and_increment_sequence_counter(), parms=self.sendparms
        )

    def _get_channel_handler_func(self) -> GeckoGetChannelProtocolHandler:
        assert self._protocol is not None
        return GeckoGetChannelProtocolHandler.request(
            self._protocol.get_and_increment_sequence_counter(),
            parms=self.sendparms,
        )

    def _get_config_file_handler_func(self) -> GeckoConfigFileProtocolHandler:
        assert self._protocol is not None
        return GeckoConfigFileProtocolHandler.request(
            self._protocol.get_and_increment_sequence_counter(),
            parms=self.sendparms,
        )

    async def _async_on_rferr(
        self, handler: GeckoRFErrProtocolHandler, sender: tuple
    ) -> None:
        await self._event_handler(GeckoSpaEvent.ERROR_RF_ERROR)
        # If we've had too many RFErr messages, then we bail waiting for user action
        if handler.total_error_count > GeckoConstants.MAX_RF_ERRORS_BEFORE_HALT:
            _LOGGER.error(
                (
                    "Spa %s has too many RF errors, so disconnecting "
                    "awaiting user intervention"
                ),
                self.descriptor.name,
            )
            await self._event_handler(GeckoSpaEvent.ERROR_TOO_MANY_RF_ERRORS)

    async def _connect(self) -> None:
        loop = asyncio.get_running_loop()
        self._con_lost = loop.create_future()

        self._transport, _protocol = await loop.create_datagram_endpoint(
            lambda: GeckoAsyncUdpProtocol(
                self._con_lost,
                (self.descriptor.destination[0], self.descriptor.destination[1]),
            ),
            family=socket.AF_INET,
        )
        assert isinstance(_protocol, GeckoAsyncUdpProtocol)
        self._protocol = _protocol
        await asyncio.sleep(GeckoConstants.CONNECTION_STEP_PAUSE_IN_SECONDS)

        self._taskman.add_task(
            GeckoUnhandledProtocolHandler().consume(self._protocol),
            "Unhandled packet",
            "SPA",
        )
        self._taskman.add_task(
            GeckoPacketProtocolHandler(async_on_handled=self._async_on_packet).consume(
                self._protocol
            ),
            "Packet handler",
            "SPA",
        )
        self._taskman.add_task(
            GeckoAsyncPartialStatusBlockProtocolHandler(
                self._protocol, async_on_handled=self._async_on_partial_status_update
            ).consume(self._protocol),
            "Partial status block handler",
            "SPA",
        )
        self._taskman.add_task(
            GeckoRFErrProtocolHandler(async_on_handled=self._async_on_rferr).consume(
                self._protocol
            ),
            "RFErr handler",
            "SPA",
        )
        self._taskman.add_task(
            GeckoWatercareErrorHandler(async_on_handled=self._async_on_wcerr).consume(
                self._protocol
            ),
            "WCErr handler",
            "SPA",
        )
        self._taskman.add_task(self._ping_loop(), "Ping loop", "SPA")
        self._taskman.add_task(self._refresh_loop(), "Refresh loop", "SPA")
        await asyncio.sleep(GeckoConstants.CONNECTION_STEP_PAUSE_IN_SECONDS)

        version_handler = await self._protocol.get(self._get_version_handler_func)
        if version_handler is None:
            await self._event_handler(
                GeckoSpaEvent.CONNECTION_PROTOCOL_RETRY_COUNT_EXCEEDED
            )
            return

        self.intouch_version_en = "{0} v{1}.{2}".format(
            version_handler.en_build, version_handler.en_major, version_handler.en_minor
        )
        self.intouch_version_co = "{0} v{1}.{2}".format(
            version_handler.co_build, version_handler.co_major, version_handler.co_minor
        )
        _LOGGER.debug(
            "Got in.touch2 firmware version EN(Home) %s/CO(Spa) %s, now get channel",
            self.intouch_version_en,
            self.intouch_version_co,
        )
        await self._event_handler(
            GeckoSpaEvent.CONNECTION_GOT_FIRMWARE_VERSION,
            intouch_version_en=self.intouch_version_en,
            intouch_version_co=self.intouch_version_co,
        )
        await asyncio.sleep(GeckoConstants.CONNECTION_STEP_PAUSE_IN_SECONDS)

        get_channel_handler = await self._protocol.get(self._get_channel_handler_func)
        if get_channel_handler is None:
            await self._event_handler(
                GeckoSpaEvent.CONNECTION_PROTOCOL_RETRY_COUNT_EXCEEDED
            )
            return

        self.channel = get_channel_handler.channel
        self.signal = get_channel_handler.signal_strength
        _LOGGER.debug("Got channel %s/%s, now get config", self.channel, self.signal)
        await self._event_handler(
            GeckoSpaEvent.CONNECTION_GOT_CHANNEL,
            channel=self.channel,
            signal=self.signal,
        )
        await asyncio.sleep(GeckoConstants.CONNECTION_STEP_PAUSE_IN_SECONDS)

        config_file_handler = await self._protocol.get(
            self._get_config_file_handler_func
        )
        if config_file_handler is None:
            await self._event_handler(
                GeckoSpaEvent.CONNECTION_PROTOCOL_RETRY_COUNT_EXCEEDED
            )
            return

        # Stash the config and log structure declarations
        self.config_version = config_file_handler.config_version
        self.log_version = config_file_handler.log_version
        plateform_key = config_file_handler.plateform_key.lower()

        _LOGGER.debug(
            "Got %s config version %s and log version %s, check if we can support this",
            plateform_key,
            self.config_version,
            self.log_version,
        )
        await self._event_handler(
            GeckoSpaEvent.CONNECTION_GOT_CONFIG_FILES,
            plateform_key=plateform_key,
            config_version=self.config_version,
            log_version=self.log_version,
        )
        await asyncio.sleep(GeckoConstants.CONNECTION_STEP_PAUSE_IN_SECONDS)

        pack_module_name = f"geckolib.driver.packs.{plateform_key}"
        try:
            GeckoPack = importlib.import_module(pack_module_name).GeckoPack
            self.pack_class = GeckoPack(self.struct)
            # TODO: Need to add base classes and type hinting to auto-generated
            # pack code and remove type-hint suppression below
            self.pack_type = self.pack_class.type  # type: ignore
        except ModuleNotFoundError:
            await self._event_handler(
                GeckoSpaEvent.CONNECTION_CANNOT_FIND_SPA_PACK,
                pack_module_name=pack_module_name,
            )
            _LOGGER.error(
                "Cannot find spa pack for %s", config_file_handler.plateform_key
            )
            return

        config_module_name = (
            f"geckolib.driver.packs.{plateform_key}-cfg-{self.config_version}"
        )
        try:
            GeckoConfigStruct = importlib.import_module(
                config_module_name
            ).GeckoConfigStruct
            self.config_class = GeckoConfigStruct(self.struct)
        except ModuleNotFoundError:
            await self._event_handler(
                GeckoSpaEvent.CONNECTION_CANNOT_FIND_CONFIG_VERSION,
                config_version=self.config_version,
            )
            _LOGGER.error(
                "Cannot find GeckoConfigStruct module for %s v%s",
                config_file_handler.plateform_key,
                self.config_version,
            )
            return

        log_module_name = (
            f"geckolib.driver.packs.{plateform_key}-log-{self.log_version}"
        )
        try:
            GeckoLogStruct = importlib.import_module(log_module_name).GeckoLogStruct
            self.log_class = GeckoLogStruct(self.struct)
        except ModuleNotFoundError:
            await self._event_handler(
                GeckoSpaEvent.CONNECTION_CANNOT_FIND_LOG_VERSION,
                log_version=self.log_version,
            )
            _LOGGER.error(
                "Cannot find GeckoLogStruct module for %s v%s",
                config_file_handler.plateform_key,
                self.log_version,
            )
            return

        _LOGGER.debug(
            "Got spa configuration Type %s - CFG %s/LOG %s, now ask for initial "
            "status block",
            self.pack_type,
            self.config_version,
            self.log_version,
        )

        await self._event_handler(GeckoSpaEvent.CONNECTION_INITIAL_DATA_BLOCK_REQUEST)
        await asyncio.sleep(GeckoConstants.CONNECTION_STEP_PAUSE_IN_SECONDS)

        if not await self.struct.get(
            self._protocol,
            lambda: GeckoStatusBlockProtocolHandler.full_request(
                self._protocol.get_and_increment_sequence_counter(),
                parms=self.sendparms,
            ),
        ):
            await self._event_handler(
                GeckoSpaEvent.CONNECTION_PROTOCOL_RETRY_COUNT_EXCEEDED
            )

            return

        _LOGGER.debug("Status block was completed, so spa can be connected")

        self.struct.build_accessors(self.config_class, self.log_class)

        self.pack = self.accessors[GeckoConstants.KEY_PACK_TYPE].value
        self.version = "{0} v{1}.{2}".format(
            self.accessors[GeckoConstants.KEY_PACK_CONFIG_ID].value,
            self.accessors[GeckoConstants.KEY_PACK_CONFIG_REV].value,
            self.accessors[GeckoConstants.KEY_PACK_CONFIG_REL].value,
        )
        self.config_number = self.accessors[GeckoConstants.KEY_CONFIG_NUMBER].value

        self._is_connected = True
        await self._event_handler(GeckoSpaEvent.CONNECTION_SPA_COMPLETE)
        _LOGGER.debug("Spa connected")

    async def connect(self) -> None:
        """Connection wrapper with exception safety"""
        try:
            await self._connect()
        except:  # noqa
            _LOGGER.exception("Exception during spa connection")
            raise

    async def disconnect(self) -> None:
        """Disconnect the spa from the async protocol"""
        self._is_connected = False
        await self._event_handler(GeckoSpaEvent.RUNNING_SPA_DISCONNECTED)
        self.struct.reset()
        self._taskman.cancel_key_tasks("SPA")
        if self._protocol is not None:
            self._protocol.disconnect()
            self._protocol = None
        self._transport = None

    @property
    def isopen(self) -> bool:
        """Return True if the spa has a UDP socket with the in.touch EN module"""
        if self._protocol is None:
            return False
        return self._protocol.isopen

    @property
    def is_responding_to_pings(self) -> bool:
        if self._last_ping is None:
            return False
        # Allow twice as much time as ping frequency to make sure boundary tests
        # are successful
        return (
            time.monotonic() - self._last_ping
        ) < GeckoConstants.PING_FREQUENCY_IN_SECONDS * 2

    async def _async_on_packet(
        self, handler: GeckoPacketProtocolHandler, sender: tuple
    ) -> None:
        if handler.parms == self.sendparms:
            assert self._protocol is not None
            self._protocol.datagram_received(handler.packet_content, handler.parms)
        else:
            _LOGGER.warning(
                "Dropping packet from %s because it didn't match %s",
                handler.parms,
                self.sendparms,
            )

    @property
    def last_ping_at(self) -> Optional[datetime]:
        """The datetime of the last successful ping response or None"""
        return self._last_ping_at

    async def _ping_loop(self) -> None:
        _LOGGER.debug("Ping loop started")

        self._last_ping = time.monotonic()

        try:
            while self.isopen:

                assert self._protocol is not None
                ping_handler = await self._protocol.get(
                    lambda: GeckoPingProtocolHandler.request(
                        parms=self.sendparms,
                        timeout=GeckoConstants.PING_FREQUENCY_IN_SECONDS,
                    ),
                    None,
                    1,
                )

                if ping_handler is not None:
                    self._last_ping = time.monotonic()
                    self._last_ping_at = datetime.utcnow().replace(tzinfo=timezone.utc)
                    self._on_change()
                    await self._event_handler(
                        GeckoSpaEvent.RUNNING_PING_RECEIVED,
                        last_ping_at=self._last_ping_at,
                    )
                else:
                    await self._event_handler(
                        GeckoSpaEvent.RUNNING_PING_MISSED,
                        last_ping_at=self._last_ping_at,
                    )
                    if (
                        time.monotonic() - self._last_ping
                        > GeckoConstants.PING_DEVICE_NOT_RESPONDING_TIMEOUT
                    ):
                        await self._event_handler(
                            GeckoSpaEvent.RUNNING_PING_NO_RESPONSE,
                            last_ping_at=self._last_ping_at,
                        )

                await asyncio.sleep(GeckoConstants.PING_FREQUENCY_IN_SECONDS)

        except asyncio.CancelledError:
            _LOGGER.debug("Ping loop cancelled")
            raise

        except:  # noqa
            _LOGGER.exception("Exception in ping loop")
            raise

    def _get_status_block_handler_func(self) -> GeckoStatusBlockProtocolHandler:
        assert self._protocol is not None
        assert self.log_class is not None
        return GeckoStatusBlockProtocolHandler.request(
            self._protocol.get_and_increment_sequence_counter(),
            self.log_class.begin,
            self.log_class.end,
            parms=self.sendparms,
        )

    async def _refresh_loop(self) -> None:
        """The refresh loop is simply to ensure that our understanding
        of the live parts of the spastruct are always up-to-date. We
        shouldn't need to do this, but this is belt and braces stuff"""

        _LOGGER.debug("Refresh loop started")
        while self.isopen:
            await asyncio.sleep(GeckoConstants.SPA_PACK_REFRESH_FREQUENCY_IN_SECONDS)
            if not self.is_connected:
                continue
            if not self.is_responding_to_pings:
                continue
            if not await self.struct.get(
                self._protocol, self._get_status_block_handler_func
            ):
                await self._event_handler(
                    GeckoSpaEvent.ERROR_PROTOCOL_RETRY_COUNT_EXCEEDED
                )

    async def _async_on_partial_status_update(
        self,
        handler: GeckoAsyncPartialStatusBlockProtocolHandler,
        sender: tuple,
    ) -> None:
        for change in handler.changes:
            self.struct.replace_status_block_segment(change[0], change[1])

    async def _on_async_set_value(self, pos, length, newvalue) -> None:
        assert self._protocol is not None
        if not self.is_connected:
            _LOGGER.warning("Cannot set value when spa not connected")
            return
        if not self.is_responding_to_pings:
            _LOGGER.debug("Cannot set value when spa not responding to pings")
            return

        pack_command_handler = await self._protocol.get(
            lambda: GeckoPackCommandProtocolHandler.set_value(
                self._protocol.get_and_increment_sequence_counter(),  # type: ignore
                self.pack_type,
                self.config_version,
                self.log_version,
                pos,
                length,
                newvalue,
                parms=self.sendparms,
            )
        )

        if pack_command_handler is None:
            await self._event_handler(GeckoSpaEvent.ERROR_PROTOCOL_RETRY_COUNT_EXCEEDED)

    def _on_set_value(self, pos, length, newvalue) -> None:
        self._taskman.add_task(
            self._on_async_set_value(pos, length, newvalue), "Set value task", "SPA"
        )

    @property
    def accessors(self) -> dict:
        return self.struct.accessors

    async def async_press(self, keypad) -> None:
        """Simulate a button press async"""
        assert self._protocol is not None
        if not self.is_connected:
            _LOGGER.warning("Cannot press keypad when spa not connected")
            return
        if not self.is_responding_to_pings:
            _LOGGER.debug("Cannot press keypad when spa not responding to pings")
            return

        pack_command_handler = await self._protocol.get(
            lambda: GeckoPackCommandProtocolHandler.keypress(
                self._protocol.get_and_increment_sequence_counter(),  # type: ignore
                self.pack_type,
                keypad,
                parms=self.sendparms,
            )
        )

        if pack_command_handler is None:
            await self._event_handler(GeckoSpaEvent.ERROR_PROTOCOL_RETRY_COUNT_EXCEEDED)

    def press(self, keypad) -> None:
        """Simulate a button press"""
        self._taskman.add_task(self.async_press(keypad), "Button press task", "SPA")

    def _get_watercare_handler_func(self) -> GeckoWatercareProtocolHandler:
        assert self._protocol is not None
        return GeckoWatercareProtocolHandler.request(
            self._protocol.get_and_increment_sequence_counter(),
            parms=self.sendparms,
        )

    async def _async_on_wcerr(
        self, handler: GeckoWatercareProtocolHandler, sender: tuple
    ) -> None:
        await self._event_handler(GeckoSpaEvent.RUNNING_SPA_WATER_CARE_ERROR)

    async def async_get_watercare(self) -> Optional[int]:
        if not self.is_connected:
            _LOGGER.warning("Cannot get watercare when spa not connected")
            return 0
        if not self.is_responding_to_pings:
            _LOGGER.debug("Cannot get watercare when spa not responding to pings")
            return 0

        assert self._protocol is not None
        get_watercare_handler = await self._protocol.get(
            self._get_watercare_handler_func
        )

        if get_watercare_handler is None:
            await self._event_handler(GeckoSpaEvent.ERROR_PROTOCOL_RETRY_COUNT_EXCEEDED)
            return None

        return get_watercare_handler.mode

    async def async_set_watercare(self, new_mode) -> None:
        if not self.is_connected:
            _LOGGER.warning("Cannot set watercare when spa not connected")
            return
        if not self.is_responding_to_pings:
            _LOGGER.debug("Cannot set watercare when spa not responding to pings")
            return

        assert self._protocol is not None
        set_watercare_handler = await self._protocol.get(
            lambda: GeckoWatercareProtocolHandler.set(
                self._protocol.get_and_increment_sequence_counter(),  # type: ignore
                new_mode,
                parms=self.sendparms,
            )
        )

        if set_watercare_handler is None:
            await self._event_handler(GeckoSpaEvent.ERROR_PROTOCOL_RETRY_COUNT_EXCEEDED)



#!/usr/bin/python3
"""
    GeckoLogStruct - A class to manage the LogStruct for 'MrSteam v1'
"""

from . import (
    GeckoByteStructAccessor,
    GeckoWordStructAccessor,
    GeckoTimeStructAccessor,
    GeckoBoolStructAccessor,
    GeckoEnumStructAccessor,
    GeckoTempStructAccessor,
)

# Constants for this class
AHEOCT = "".join(chr(c) for c in [72, 50, 79, 50, 69, 114, 114])
AKQXPI = "".join(chr(c) for c in [72, 111, 117, 114, 115])
AONPYY = 279
ASSAKQ = 263
BFEGZU = "".join(chr(c) for c in [105, 110, 84, 101, 114, 102, 97, 99, 101])
BMJVHF = "".join(chr(c) for c in [79, 78])
BSKSOK = "".join(chr(c) for c in [74, 117, 109, 112, 101, 114, 51])
CBFEGZ = "".join(chr(c) for c in [75, 54, 48, 48])
CQBMJV = 257
CTHBSK = "".join(chr(c) for c in [74, 117, 109, 112, 101, 114, 57])
CVYYPI = 3
CWAONP = 278
CXQIEF = "".join(chr(c) for c in [78, 111, 75, 101, 121, 112, 97, 100])
ECVYYP = "".join(chr(c) for c in [67, 104, 114, 111, 109, 97, 83, 116, 97, 116, 101])
EFXQGL = "".join(chr(c) for c in [83, 108, 97, 118, 101, 77, 111, 100, 101])
EGZUQE = "".join(chr(c) for c in [105, 110, 89, 84])
EKCWAO = 277
EOCTHB = 4
EXLSXU = "".join(chr(c) for c in [80, 97, 99, 107, 77, 101, 109, 82, 97, 110, 103, 101])
FEGZUQ = "".join(chr(c) for c in [105, 110, 84, 111, 117, 99, 104])
FTHECV = "".join(chr(c) for c in [80, 97, 117, 115, 101])
FXQGLR = 259
GLRAHE = 260
GZUQEX = "".join(
    chr(c) for c in [67, 111, 108, 111, 114, 95, 75, 101, 121, 112, 97, 100]
)
HBSKSO = "".join(chr(c) for c in [74, 117, 109, 112, 101, 114, 50])
HECVYY = "".join(chr(c) for c in [68, 105, 97, 103, 110, 111, 115, 116, 105, 99])
HEOCTH = "".join(chr(c) for c in [83, 108, 97, 118, 101, 72, 50, 79, 50, 69, 114, 114])
HFTHEC = "".join(chr(c) for c in [65, 76, 76])
HIUSOO = 271
HUOJRJ = 7
ICXQIE = 2658
IEFXQG = 5
IPIVLA = "".join(chr(c) for c in [80, 114, 111, 103, 50])
IUSOOQ = "".join(chr(c) for c in [80, 97, 99, 107, 66, 111, 111, 116, 82, 101, 108])
IUXFEF = 280
IVLASS = "".join(
    chr(c) for c in [85, 115, 101, 114, 83, 101, 116, 112, 111, 105, 110, 116]
)
JHIUSO = "".join(chr(c) for c in [80, 97, 99, 107, 66, 111, 111, 116, 82, 101, 118])
JMCBFE = "".join(chr(c) for c in [105, 110, 67, 108, 101, 97, 114])
JRJHIU = "".join(chr(c) for c in [80, 97, 99, 107, 66, 111, 111, 116, 73, 68])
JVHFTH = 0
KCWAON = "".join(chr(c) for c in [80, 97, 99, 107, 67, 111, 114, 101, 82, 101, 108])
KPHUOJ = 6
KQXPIC = 256
KSOKPH = "".join(chr(c) for c in [74, 117, 109, 112, 101, 114, 53])
LASSAK = "".join(chr(c) for c in [85, 115, 101, 114, 82, 117, 110, 116, 105, 109, 101])
LIUXFE = 256
LRAHEO = "".join(chr(c) for c in [80, 114, 114, 50, 69, 114, 114])
LSXUJU = "".join(chr(c) for c in [49, 54, 75])
MCBFEG = "".join(chr(c) for c in [105, 110, 88, 77])
NPYYLI = 280
NRSJMC = "".join(chr(c) for c in [77, 97, 115, 73, 66, 67])
OCTHBS = "".join(chr(c) for c in [75, 101, 121, 83, 116, 117, 99, 107, 69, 114, 114])
OJRJHI = 267
OKPHUO = "".join(chr(c) for c in [74, 117, 109, 112, 101, 114, 55])
ONPYYL = "".join(
    chr(c) for c in [80, 97, 99, 107, 83, 116, 97, 116, 117, 115, 76, 105, 98]
)
OOQNRS = 273
OQNRSJ = "".join(chr(c) for c in [85, 110, 107, 110, 111, 119, 110])
PHUOJR = "".join(chr(c) for c in [74, 117, 109, 112, 101, 114, 56])
PICXQI = "".join(
    chr(c) for c in [87, 97, 116, 101, 114, 68, 101, 116, 101, 99, 116, 101, 100]
)
PIPIVL = "".join(chr(c) for c in [80, 114, 111, 103, 49])
QBMJVH = "".join(chr(c) for c in [79, 70, 70])
QGLRAH = "".join(
    chr(c) for c in [80, 111, 119, 101, 114, 70, 97, 105, 108, 69, 114, 114]
)
QIEFXQ = "".join(
    chr(c) for c in [69, 120, 112, 114, 101, 115, 115, 67, 121, 99, 108, 101]
)
QNRSJM = "".join(chr(c) for c in [105, 110, 88, 69])
QXPICX = "".join(
    chr(c) for c in [69, 120, 116, 101, 114, 110, 97, 108, 80, 114, 111, 98, 101]
)
RAHEOC = "".join(chr(c) for c in [80, 114, 114, 49, 69, 114, 114])
RJHIUS = 269
RSJMCB = "".join(chr(c) for c in [77, 73, 65])
SAKQXP = 265
SJMCBF = "".join(chr(c) for c in [68, 74, 83, 52])
SKSOKP = "".join(chr(c) for c in [74, 117, 109, 112, 101, 114, 52])
SOKPHU = "".join(chr(c) for c in [74, 117, 109, 112, 101, 114, 54])
SOOQNR = "".join(chr(c) for c in [80, 97, 99, 107, 84, 121, 112, 101])
SSAKQX = "".join(chr(c) for c in [85, 115, 101, 114, 65, 114, 111, 109, 97])
SXUJUT = "".join(chr(c) for c in [51, 50, 75])
THBSKS = 266
THECVY = 1
TYEKCW = 275
UJUTYE = "".join(chr(c) for c in [54, 52, 75])
UOJRJH = "".join(chr(c) for c in [77, 97, 120, 82, 117, 110, 116, 105, 109, 101])
UQEXLS = "".join(chr(c) for c in [77, 114, 83, 116, 101, 97, 109])
USOOQN = 272
UTYEKC = "".join(chr(c) for c in [80, 97, 99, 107, 67, 111, 114, 101, 73, 68])
VHFTHE = 2
VLASSA = 262
VYYPIP = "".join(
    chr(c) for c in [83, 101, 108, 101, 99, 116, 101, 100, 80, 114, 111, 103]
)
WAONPY = "".join(
    chr(c) for c in [80, 97, 99, 107, 67, 111, 110, 102, 105, 103, 76, 105, 98]
)
XLSXUJ = 274
XPICXQ = 258
XQGLRA = "".join(
    chr(c)
    for c in [83, 108, 97, 118, 101, 72, 101, 97, 116, 101, 114, 83, 116, 97, 116, 101]
)
XQIEFX = "".join(
    chr(c) for c in [78, 111, 82, 101, 103, 117, 108, 97, 116, 105, 111, 110]
)
XUJUTY = "".join(chr(c) for c in [52, 56, 75])
YEKCWA = "".join(chr(c) for c in [80, 97, 99, 107, 67, 111, 114, 101, 82, 101, 118])
YPIPIV = "".join(chr(c) for c in [85, 115, 101, 114])
YYLIUX = "".join(chr(c) for c in [76, 73])
YYPIPI = 261
ZCQBMJ = "".join(chr(c) for c in [77, 111, 100, 101])
ZUQEXL = "".join(chr(c) for c in [105, 110, 89, 74])
JUTYEK = [LSXUJU, SXUJUT, XUJUTY, UJUTYE]
MJVHFT = [QBMJVH, BMJVHF]
PIVLAS = [YPIPIV, PIPIVL, IPIVLA]
PYYLIU = []
QEXLSX = [
    OQNRSJ,
    QNRSJM,
    NRSJMC,
    RSJMCB,
    SJMCBF,
    JMCBFE,
    MCBFEG,
    CBFEGZ,
    BFEGZU,
    FEGZUQ,
    EGZUQE,
    GZUQEX,
    ZUQEXL,
    UQEXLS,
]
YLIUXF = [YYLIUX]


class GeckoLogStruct:
    def __init__(self, struct_):
        self.struct = struct_

    @property
    def version(self):
        return THECVY

    @property
    def begin(self):
        return LIUXFE

    @property
    def end(self):
        return IUXFEF

    @property
    def all_device_keys(self):
        return YLIUXF

    @property
    def user_demand_keys(self):
        return PYYLIU

    @property
    def accessors(self):
        return {
            ZCQBMJ: GeckoEnumStructAccessor(
                self.struct, ZCQBMJ, CQBMJV, JVHFTH, MJVHFT, None, VHFTHE, HFTHEC
            ),
            FTHECV: GeckoBoolStructAccessor(
                self.struct, FTHECV, CQBMJV, THECVY, HFTHEC
            ),
            HECVYY: GeckoBoolStructAccessor(
                self.struct, HECVYY, CQBMJV, VHFTHE, HFTHEC
            ),
            ECVYYP: GeckoEnumStructAccessor(
                self.struct, ECVYYP, CQBMJV, CVYYPI, MJVHFT, None, VHFTHE, HFTHEC
            ),
            VYYPIP: GeckoEnumStructAccessor(
                self.struct, VYYPIP, YYPIPI, None, PIVLAS, None, None, HFTHEC
            ),
            IVLASS: GeckoByteStructAccessor(self.struct, IVLASS, VLASSA, HFTHEC),
            LASSAK: GeckoWordStructAccessor(self.struct, LASSAK, ASSAKQ, HFTHEC),
            SSAKQX: GeckoEnumStructAccessor(
                self.struct, SSAKQX, SAKQXP, None, MJVHFT, None, None, HFTHEC
            ),
            AKQXPI: GeckoByteStructAccessor(self.struct, AKQXPI, KQXPIC, None),
            QXPICX: GeckoBoolStructAccessor(self.struct, QXPICX, XPICXQ, JVHFTH, None),
            PICXQI: GeckoBoolStructAccessor(self.struct, PICXQI, ICXQIE, THECVY, None),
            CXQIEF: GeckoBoolStructAccessor(self.struct, CXQIEF, XPICXQ, VHFTHE, None),
            XQIEFX: GeckoBoolStructAccessor(self.struct, XQIEFX, XPICXQ, CVYYPI, None),
            QIEFXQ: GeckoBoolStructAccessor(self.struct, QIEFXQ, XPICXQ, IEFXQG, None),
            EFXQGL: GeckoEnumStructAccessor(
                self.struct, EFXQGL, FXQGLR, THECVY, MJVHFT, None, VHFTHE, None
            ),
            XQGLRA: GeckoEnumStructAccessor(
                self.struct, XQGLRA, FXQGLR, CVYYPI, MJVHFT, None, VHFTHE, None
            ),
            QGLRAH: GeckoBoolStructAccessor(self.struct, QGLRAH, GLRAHE, JVHFTH, None),
            LRAHEO: GeckoBoolStructAccessor(self.struct, LRAHEO, GLRAHE, THECVY, None),
            RAHEOC: GeckoBoolStructAccessor(self.struct, RAHEOC, GLRAHE, VHFTHE, None),
            AHEOCT: GeckoBoolStructAccessor(self.struct, AHEOCT, GLRAHE, CVYYPI, None),
            HEOCTH: GeckoBoolStructAccessor(self.struct, HEOCTH, GLRAHE, EOCTHB, None),
            OCTHBS: GeckoBoolStructAccessor(self.struct, OCTHBS, GLRAHE, IEFXQG, None),
            CTHBSK: GeckoBoolStructAccessor(self.struct, CTHBSK, THBSKS, JVHFTH, None),
            HBSKSO: GeckoBoolStructAccessor(self.struct, HBSKSO, THBSKS, THECVY, None),
            BSKSOK: GeckoBoolStructAccessor(self.struct, BSKSOK, THBSKS, VHFTHE, None),
            SKSOKP: GeckoBoolStructAccessor(self.struct, SKSOKP, THBSKS, CVYYPI, None),
            KSOKPH: GeckoBoolStructAccessor(self.struct, KSOKPH, THBSKS, EOCTHB, None),
            SOKPHU: GeckoBoolStructAccessor(self.struct, SOKPHU, THBSKS, IEFXQG, None),
            OKPHUO: GeckoBoolStructAccessor(self.struct, OKPHUO, THBSKS, KPHUOJ, None),
            PHUOJR: GeckoBoolStructAccessor(self.struct, PHUOJR, THBSKS, HUOJRJ, None),
            UOJRJH: GeckoWordStructAccessor(self.struct, UOJRJH, OJRJHI, None),
            JRJHIU: GeckoWordStructAccessor(self.struct, JRJHIU, RJHIUS, None),
            JHIUSO: GeckoByteStructAccessor(self.struct, JHIUSO, HIUSOO, None),
            IUSOOQ: GeckoByteStructAccessor(self.struct, IUSOOQ, USOOQN, None),
            SOOQNR: GeckoEnumStructAccessor(
                self.struct, SOOQNR, OOQNRS, None, QEXLSX, None, None, None
            ),
            EXLSXU: GeckoEnumStructAccessor(
                self.struct, EXLSXU, XLSXUJ, JVHFTH, JUTYEK, None, EOCTHB, None
            ),
            UTYEKC: GeckoWordStructAccessor(self.struct, UTYEKC, TYEKCW, None),
            YEKCWA: GeckoByteStructAccessor(self.struct, YEKCWA, EKCWAO, None),
            KCWAON: GeckoByteStructAccessor(self.struct, KCWAON, CWAONP, None),
            WAONPY: GeckoByteStructAccessor(self.struct, WAONPY, AONPYY, None),
            ONPYYL: GeckoByteStructAccessor(self.struct, ONPYYL, NPYYLI, None),
        }



#!/usr/bin/python3
"""
    GeckoConfigStruct - A class to manage the ConfigStruct for 'InXE v58'
"""

from . import (
    GeckoByteStructAccessor,
    GeckoWordStructAccessor,
    GeckoTimeStructAccessor,
    GeckoBoolStructAccessor,
    GeckoEnumStructAccessor,
    GeckoTempStructAccessor,
)

# Constants for this class
ACCPQI = "".join(chr(c) for c in [79, 51, 80, 117, 109, 112])
ACQFFT = "".join(chr(c) for c in [72, 121, 100, 114, 111, 112, 111, 111, 108])
AHEOCT = "".join(chr(c) for c in [79, 117, 116, 53, 67, 117, 114])
AJVDQL = "".join(chr(c) for c in [71, 82, 69, 69, 78])
AKQXPI = "".join(chr(c) for c in [79, 117, 116, 51])
AMJMAO = 10
AONPYY = "".join(
    chr(c)
    for c in [
        67,
        112,
        79,
        102,
        102,
        84,
        105,
        109,
        101,
        68,
        117,
        114,
        105,
        110,
        103,
        79,
        84,
    ]
)
AWBSIR = 82
BDJQRJ = "".join(chr(c) for c in [83, 117, 110, 114, 97, 110, 115])
BFEGZU = "".join(
    chr(c) for c in [70, 105, 108, 116, 80, 49, 68, 117, 114, 79, 110, 108, 121]
)
BIAMJM = 8
BJEUTO = "".join(chr(c) for c in [83, 72, 79, 87, 95, 65, 76, 76, 95, 77, 83, 71])
BLKXSJ = 31
BMJVHF = "".join(chr(c) for c in [79, 117, 116, 49])
BQFYLJ = "".join(
    chr(c) for c in [77, 117, 108, 116, 105, 75, 101, 121, 79, 112, 116, 105, 111, 110]
)
BSIRYX = 72
BSKSOK = "".join(chr(c) for c in [68, 105, 114, 101, 99, 116, 67, 117, 114])
BSSUHB = "".join(chr(c) for c in [66, 117, 108, 108, 102, 114, 111, 103])
BVWVUB = "".join(chr(c) for c in [72, 111, 116, 115, 112, 114, 105, 110, 103])
BXTIAC = 7
BXYBQS = "".join(
    chr(c)
    for c in [80, 117, 109, 112, 49, 85, 115, 101, 114, 65, 99, 99, 101, 115, 115]
)
BYGDSB = "".join(chr(c) for c in [77, 65, 65, 88])
CBFEGZ = "".join(chr(c) for c in [70, 105, 108, 116, 80, 49])
CCPQIP = 29
CQBMJV = 0
CQFFTT = "".join(
    chr(c) for c in [69, 110, 100, 108, 101, 115, 115, 80, 111, 111, 108, 115]
)
CRTFMN = "".join(chr(c) for c in [84, 104, 101, 114, 109, 111, 83, 112, 97, 115])
CTHBSK = "".join(chr(c) for c in [68, 105, 114, 101, 99, 116])
CVYYPI = "".join(chr(c) for c in [80, 52, 72])
CWAONP = "".join(
    chr(c)
    for c in [67, 112, 79, 110, 84, 105, 109, 101, 68, 117, 114, 105, 110, 103, 79, 84]
)
CXQIEF = "".join(chr(c) for c in [79, 117, 116, 72, 116, 114])
DJQRJJ = "".join(chr(c) for c in [83, 117, 110, 114, 105, 115, 101])
DNIBXT = "".join(chr(c) for c in [68, 105, 115, 97, 98, 108, 101])
DQLAII = "".join(chr(c) for c in [77, 65, 71, 69, 78, 84, 65])
DSBDJQ = "".join(
    chr(c)
    for c in [80, 114, 101, 109, 105, 117, 109, 95, 76, 101, 105, 115, 117, 114, 101]
)
DUBSSU = "".join(chr(c) for c in [66, 101, 108, 108, 97, 103, 105, 111])
ECVYYP = "".join(chr(c) for c in [80, 51, 76])
EFJTAC = 28
EFXQGL = 36
EGZUQE = "".join(chr(c) for c in [67, 112, 85, 115, 97, 103, 101])
EJNIBX = "".join(chr(c) for c in [50, 52, 104])
EKCWAO = "".join(chr(c) for c in [79, 84, 84, 114, 105, 103, 103, 101, 114, 71])
EKVKZI = "".join(
    chr(c)
    for c in [67, 117, 115, 116, 111, 109, 75, 101, 121, 69, 110, 97, 98, 108, 101, 100]
)
EOCTHB = "".join(chr(c) for c in [79, 117, 116, 72, 116, 82, 67, 117, 114])
EUTOPH = "".join(
    chr(c) for c in [75, 101, 121, 112, 97, 100, 79, 112, 116, 105, 111, 110, 115, 52]
)
EXLSXU = "".join(chr(c) for c in [79, 116, 79, 112, 116, 105, 111, 110])
FCRTFM = "".join(chr(c) for c in [65, 115, 112, 101, 110])
FEFJTA = "".join(chr(c) for c in [79, 51, 85, 115, 97, 103, 101])
FFTTID = "".join(chr(c) for c in [65, 108, 112, 115])
FJBIAM = 23
FJTACC = "".join(chr(c) for c in [70, 105, 108, 116, 101, 114])
FTHECV = "".join(chr(c) for c in [80, 50, 72])
FTSIFJ = 4
FTTIDU = "".join(chr(c) for c in [65, 114, 116, 101, 115, 105, 97, 110])
FXQGLR = "".join(chr(c) for c in [79, 117, 116, 50, 67, 117, 114])
FYLJUI = "".join(
    chr(c) for c in [78, 111, 66, 108, 111, 119, 101, 114, 79, 110, 73, 50, 67]
)
GDSBDJ = "".join(chr(c) for c in [80, 68, 67])
GLRAHE = 38
GQPLSP = 74
GYOUSP = "".join(
    chr(c)
    for c in [
        77,
        97,
        120,
        78,
        117,
        109,
        98,
        101,
        114,
        79,
        102,
        80,
        104,
        97,
        115,
        101,
        115,
    ]
)
GZUQEX = 27
HBVWVU = "".join(chr(c) for c in [70, 111, 117, 114, 95, 87, 105, 110, 100])
HECVYY = "".join(chr(c) for c in [80, 51, 72])
HEOCTH = 40
HFTHEC = "".join(chr(c) for c in [80, 49, 76])
HIUSOO = 56
HTBJEU = "".join(
    chr(c) for c in [73, 110, 102, 111, 77, 115, 103, 67, 111, 110, 102, 105, 103]
)
HUGTYI = 58
HUOJRJ = 80
HXEKVK = "".join(
    chr(c)
    for c in [
        75,
        101,
        121,
        112,
        97,
        100,
        84,
        104,
        101,
        114,
        97,
        112,
        121,
        83,
        117,
        112,
        112,
        111,
        114,
        116,
    ]
)
IACQFF = "".join(chr(c) for c in [71, 101, 110, 101, 114, 105, 99])
IAMJMA = "".join(chr(c) for c in [69, 99, 111, 110, 68, 117, 114])
IBXYBQ = 64
ICXQIE = 16
IDNIBX = "".join(
    chr(c)
    for c in [
        75,
        101,
        121,
        112,
        97,
        100,
        66,
        97,
        99,
        107,
        108,
        105,
        103,
        104,
        116,
        69,
        100,
        105,
        116,
    ]
)
IDUBSS = "".join(chr(c) for c in [66, 101, 97, 99, 104, 99, 111, 109, 98, 101, 114])
IEFXQG = "".join(chr(c) for c in [79, 117, 116, 49, 67, 117, 114])
IFJBIA = "".join(chr(c) for c in [70, 105, 108, 116, 68, 117, 114, 50])
IGYOUS = 77
IIDNIB = 8
IKFWRK = "".join(chr(c) for c in [77, 97, 115, 116, 101, 114])
ILXWAJ = "".join(
    chr(c) for c in [66, 114, 101, 97, 107, 101, 114, 67, 104, 97, 110, 103, 101]
)
INEJNI = 34
IPIVLA = "".join(chr(c) for c in [79, 51])
IPOUYN = "".join(chr(c) for c in [83, 116, 97, 110, 100, 97, 114, 100])
IRYXBQ = "".join(
    chr(c) for c in [79, 102, 102, 79, 110, 67, 117, 115, 116, 111, 109, 75, 101, 121]
)
IUSOOQ = "".join(chr(c) for c in [76, 49, 50, 48, 84, 105, 109, 101, 114])
IUXFEF = 78
IVLASS = "".join(chr(c) for c in [72, 84, 82])
JBIAMJ = "".join(chr(c) for c in [69, 99, 111, 110, 83, 116, 97, 114, 116])
JHIUSO = "".join(chr(c) for c in [76, 49, 50, 48, 84, 105, 109, 101, 79, 117, 116])
JIGYOU = "".join(
    chr(c) for c in [78, 111, 72, 101, 97, 116, 80, 101, 114, 105, 111, 100]
)
JJJVYF = "".join(
    chr(c) for c in [87, 87, 79, 95, 87, 104, 105, 114, 108, 99, 97, 114, 101]
)
JJVYFC = "".join(chr(c) for c in [79, 107, 101, 97, 110, 111, 115])
JMAOAW = 71
JMCBFE = "".join(chr(c) for c in [80, 117, 114, 103, 101, 79, 110, 108, 121])
JQRJJJ = "".join(
    chr(c) for c in [83, 117, 112, 101, 114, 105, 111, 114, 83, 112, 97, 115]
)
JRJHIU = "".join(
    chr(c) for c in [76, 105, 103, 104, 116, 84, 105, 109, 101, 79, 117, 116]
)
JTACCP = "".join(chr(c) for c in [65, 108, 119, 97, 121, 115])
JUIKFW = "".join(chr(c) for c in [77, 97, 115, 116, 101, 114, 83, 108, 97, 118, 101])
JUTYEK = "".join(chr(c) for c in [80, 117, 114, 103, 101, 83, 112, 101, 101, 100])
JVDQLA = "".join(chr(c) for c in [89, 69, 76, 76, 79, 87])
JVHFTH = "".join(chr(c) for c in [78, 65])
JVYFCR = "".join(chr(c) for c in [67, 108, 101, 97, 114, 119, 97, 116, 101, 114])
JWMNZM = "".join(
    chr(c) for c in [77, 97, 120, 83, 101, 116, 112, 111, 105, 110, 116, 71]
)
JYKLGQ = 52
JYMOUN = "".join(chr(c) for c in [84, 101, 109, 112, 85, 110, 105, 116, 115])
KCWAON = 58
KFWRKI = "".join(chr(c) for c in [83, 108, 97, 118, 101])
KINEJN = "".join(chr(c) for c in [84, 105, 109, 101, 70, 111, 114, 109, 97, 116])
KLGQPL = 53
KQXPIC = 14
KSOKPH = "".join(chr(c) for c in [79, 117, 116, 76, 105])
KVKZIL = "".join(
    chr(c) for c in [67, 111, 110, 102, 105, 103, 67, 104, 97, 110, 103, 101]
)
KXSJWM = 35
KZILXW = "".join(
    chr(c)
    for c in [80, 65, 83, 83, 87, 79, 82, 68, 95, 80, 82, 79, 84, 69, 67, 84, 69, 68]
)
LAIIDN = "".join(chr(c) for c in [87, 72, 73, 84, 69])
LASSAK = "".join(chr(c) for c in [70, 117, 108, 108, 79, 110])
LGQPLS = "".join(chr(c) for c in [73, 110, 112, 117, 116, 77, 101, 110, 117])
LIUXFE = "".join(chr(c) for c in [68, 114, 97, 105, 110, 77, 111, 100, 101])
LKXSJW = "".join(
    chr(c) for c in [67, 111, 111, 108, 100, 111, 119, 110, 84, 105, 109, 101]
)
LNMHXE = "".join(chr(c) for c in [76, 97, 115, 116, 80, 117, 109, 112, 75, 101, 121])
LRAHEO = "".join(chr(c) for c in [79, 117, 116, 52, 67, 117, 114])
LSPFTS = "".join(chr(c) for c in [70, 105, 108, 116, 70, 114, 101, 113])
LSXUJU = "".join(chr(c) for c in [68, 105, 115, 97, 98, 108, 101, 100])
LXWAJV = "".join(
    chr(c)
    for c in [
        75,
        101,
        121,
        112,
        97,
        100,
        66,
        97,
        99,
        107,
        108,
        105,
        103,
        104,
        116,
        67,
        111,
        108,
        111,
        114,
    ]
)
MAOAWB = "".join(
    chr(c) for c in [79, 85, 84, 83, 73, 68, 69, 95, 70, 73, 76, 84, 69, 82]
)
MCBFEG = "".join(chr(c) for c in [70, 105, 108, 116, 67, 80])
MHXEKV = 5
MJMAOA = "".join(
    chr(c)
    for c in [
        69,
        99,
        111,
        110,
        80,
        114,
        111,
        103,
        65,
        118,
        97,
        105,
        108,
        97,
        98,
        108,
        101,
    ]
)
MJVHFT = 12
MNHTBJ = "".join(
    chr(c)
    for c in [
        81,
        117,
        105,
        99,
        107,
        79,
        102,
        102,
        75,
        101,
        121,
        69,
        110,
        97,
        98,
        108,
        101,
    ]
)
MNZMJI = "".join(chr(c) for c in [69, 99, 111, 110, 84, 121, 112, 101])
MOUNBL = "".join(chr(c) for c in [70])
NBLKXS = "".join(chr(c) for c in [72, 101, 97, 116, 101, 114, 80, 117, 109, 112])
NEJNIB = "".join(chr(c) for c in [65, 109, 80, 109])
NHTBJE = 45
NIBXTI = "".join(chr(c) for c in [69, 110, 97, 98, 108, 101])
NIBXYB = "".join(
    chr(c)
    for c in [65, 109, 98, 105, 97, 110, 116, 79, 72, 84, 114, 105, 103, 65, 68, 67]
)
NPYYLI = "".join(
    chr(c)
    for c in [
        70,
        105,
        108,
        116,
        79,
        110,
        84,
        105,
        109,
        101,
        68,
        117,
        114,
        105,
        110,
        103,
        79,
        84,
    ]
)
NQJYMO = "".join(chr(c) for c in [83, 101, 116, 112, 111, 105, 110, 116, 71])
NQLNMH = "".join(
    chr(c)
    for c in [66, 108, 111, 119, 101, 114, 75, 101, 121, 79, 112, 116, 105, 111, 110]
)
NRSJMC = 2
NZMJIG = 70
OAWBSI = "".join(chr(c) for c in [85, 68, 80, 114, 111, 103, 69, 99, 111, 110])
OCTHBS = 42
OJRJHI = 54
OKPHUO = "".join(chr(c) for c in [76, 73])
ONPYYL = 60
OOQNRS = "".join(chr(c) for c in [79, 119, 110])
OUNBLK = "".join(chr(c) for c in [67])
OUSPBW = "".join(chr(c) for c in [85, 76, 95, 67, 69])
PBWJYK = "".join(chr(c) for c in [67, 69])
PFTSIF = "".join(chr(c) for c in [70, 105, 108, 116, 83, 116, 97, 114, 116])
PHUOJR = "".join(chr(c) for c in [76, 105, 103, 104, 116, 73, 110, 116, 115])
PICXQI = "".join(chr(c) for c in [79, 117, 116, 53])
PIPIVL = "".join(chr(c) for c in [67, 80])
PIVLAS = "".join(chr(c) for c in [76, 49, 50, 48])
POUYNQ = "".join(chr(c) for c in [84, 111, 103, 103, 108, 101])
PQIPOU = "".join(chr(c) for c in [79, 51, 84, 121, 112, 101])
PYYLIU = 61
QBMJVH = "".join(chr(c) for c in [65, 76, 76])
QFFTTI = "".join(chr(c) for c in [87, 101, 108, 108, 105, 115])
QFYLJU = 76
QGLRAH = "".join(chr(c) for c in [79, 117, 116, 51, 67, 117, 114])
QIPOUY = 30
QJYMOU = 1
QLAIID = "".join(chr(c) for c in [67, 89, 65, 78])
QLNMHX = "".join(chr(c) for c in [70, 114, 101, 101, 80, 117, 109, 112, 75, 101, 121])
QNRSJM = 0
QPLSPF = "".join(chr(c) for c in [68, 117, 97, 108, 80, 97, 99, 107])
QRJJJV = "".join(
    chr(c) for c in [83, 112, 97, 95, 73, 110, 100, 117, 115, 116, 114, 105, 101, 115]
)
QSNQLN = "".join(
    chr(c) for c in [83, 101, 108, 102, 67, 108, 101, 97, 110, 77, 115, 103]
)
QXPICX = "".join(chr(c) for c in [79, 117, 116, 52])
RAHEOC = 39
RJHIUS = 55
RJJJVY = "".join(chr(c) for c in [86, 105, 107, 105, 110, 103])
RKINEJ = 75
RSJMCB = "".join(
    chr(c) for c in [70, 105, 108, 116, 73, 110, 116, 101, 114, 102, 97, 99, 101]
)
RTFMNH = "".join(chr(c) for c in [84, 105, 116, 97, 110, 95, 83, 112, 97, 115])
RYXBQF = 1
SAKQXP = 13
SBDJQR = "".join(chr(c) for c in [83, 116, 114, 111, 110, 103])
SIFJBI = 6
SIRYXB = "".join(
    chr(c)
    for c in [83, 111, 97, 107, 79, 110, 67, 117, 115, 116, 111, 109, 75, 101, 121]
)
SJMCBF = 32
SJWMNZ = 66
SKSOKP = 41
SNQLNM = 4
SOKPHU = 79
SOOQNR = "".join(chr(c) for c in [83, 104, 97, 114, 101, 100])
SPBWJY = "".join(chr(c) for c in [85, 76])
SPFTSI = 3
SSAKQX = "".join(chr(c) for c in [79, 117, 116, 50])
SSUHBV = "".join(chr(c) for c in [67, 111, 97, 115, 116])
SUHBVW = "".join(
    chr(c) for c in [68, 105, 109, 101, 110, 115, 105, 111, 110, 95, 111, 110, 101]
)
SXUJUT = "".join(
    chr(c) for c in [65, 108, 119, 97, 121, 115, 69, 110, 97, 98, 108, 101, 100]
)
TBJEUT = "".join(
    chr(c) for c in [72, 73, 68, 69, 95, 68, 69, 84, 65, 73, 76, 69, 68, 95, 77, 83, 71]
)
TFMNHT = "".join(chr(c) for c in [73, 98, 101, 114, 83, 112, 97])
THBSKS = 17
THECVY = "".join(chr(c) for c in [80, 50, 76])
TIACQF = 44
TIDUBS = "".join(chr(c) for c in [66, 97, 114, 101, 102, 111, 111, 116])
TSIFJB = "".join(chr(c) for c in [70, 105, 108, 116, 68, 117, 114])
TTIDUB = "".join(chr(c) for c in [65, 114, 99, 116, 105, 99])
TYEKCW = "".join(chr(c) for c in [72, 105])
UBSSUH = "".join(
    chr(c)
    for c in [76, 101, 105, 115, 117, 114, 101, 95, 80, 114, 111, 100, 95, 73, 110, 100]
)
UBYGDS = "".join(chr(c) for c in [80, 114, 111, 95, 70, 108, 111, 97, 116])
UHBVWV = "".join(chr(c) for c in [68, 121, 110, 97, 115, 116, 121])
UIKFWR = 73
UOJRJH = "".join(chr(c) for c in [80, 117, 109, 112, 84, 105, 109, 101, 79, 117, 116])
UQEXLS = "".join(chr(c) for c in [65, 76, 87, 65, 89, 83, 95, 79, 78])
USOOQN = 81
USPBWJ = 51
UTOPHU = 46
UTYEKC = "".join(chr(c) for c in [76, 111])
UXFEFJ = "".join(chr(c) for c in [80, 49])
UYNQJY = "".join(
    chr(c) for c in [79, 51, 83, 117, 115, 112, 101, 110, 100, 84, 105, 109, 101]
)
VDQLAI = "".join(chr(c) for c in [66, 76, 85, 69])
VHFTHE = "".join(chr(c) for c in [80, 49, 72])
VKZILX = "".join(
    chr(c) for c in [78, 79, 95, 82, 69, 83, 84, 82, 73, 67, 84, 73, 79, 78]
)
VLASSA = "".join(chr(c) for c in [])
VUBYGD = "".join(chr(c) for c in [76, 65])
VWVUBY = "".join(chr(c) for c in [74, 97, 99, 117, 122, 122, 105])
VYFCRT = "".join(chr(c) for c in [68, 101, 108, 117, 120, 101])
VYYPIP = "".join(chr(c) for c in [80, 52, 76])
WAJVDQ = "".join(chr(c) for c in [82, 69, 68])
WAONPY = 59
WBSIRY = "".join(
    chr(c)
    for c in [
        69,
        99,
        111,
        110,
        67,
        111,
        110,
        116,
        114,
        111,
        108,
        97,
        98,
        108,
        101,
        77,
        97,
        110,
        117,
        97,
        108,
        108,
        121,
    ]
)
WJYKLG = "".join(chr(c) for c in [78, 98, 80, 104, 97, 115, 101, 115])
WMNZMJ = 68
WRKINE = "".join(chr(c) for c in [83, 108, 97, 118, 101, 67, 111, 110, 102, 105, 103])
WVUBYG = "".join(chr(c) for c in [74, 97, 122, 122, 105])
XBQFYL = 3
XEKVKZ = 43
XLSXUJ = 57
XPICXQ = 15
XQGLRA = 37
XQIEFX = 18
XSJWMN = "".join(
    chr(c) for c in [77, 105, 110, 83, 101, 116, 112, 111, 105, 110, 116, 71]
)
XTIACQ = "".join(chr(c) for c in [67, 117, 115, 116, 111, 109, 101, 114, 73, 68])
XUJUTY = "".join(
    chr(c) for c in [87, 105, 116, 104, 83, 80, 79, 118, 101, 114, 57, 53, 70]
)
XWAJVD = "".join(chr(c) for c in [79, 70, 70])
XYBQSN = "".join(chr(c) for c in [66, 111, 116, 104, 83, 112, 101, 101, 100, 115])
YBQSNQ = "".join(
    chr(c) for c in [72, 105, 103, 104, 83, 112, 101, 101, 100, 79, 110, 108, 121]
)
YFCRTF = "".join(chr(c) for c in [73, 100, 111, 108])
YGDSBD = "".join(chr(c) for c in [77, 97, 114, 113, 117, 105, 115])
YKLGQP = "".join(
    chr(c) for c in [73, 110, 112, 117, 116, 67, 117, 114, 114, 101, 110, 116]
)
YLIUXF = 62
YLJUIK = "".join(chr(c) for c in [66, 108, 111, 119, 101, 114, 79, 110, 73, 50, 67])
YMOUNB = 33
YNQJYM = 63
YOUSPB = 26
YPIPIV = "".join(chr(c) for c in [66, 76, 79])
YXBQFY = "".join(
    chr(c)
    for c in [
        67,
        108,
        101,
        97,
        110,
        117,
        112,
        79,
        110,
        67,
        117,
        115,
        116,
        111,
        109,
        75,
        101,
        121,
    ]
)
YYLIUX = "".join(
    chr(c)
    for c in [70, 105, 108, 116, 83, 117, 115, 112, 101, 110, 100, 84, 105, 109, 101]
)
YYPIPI = "".join(chr(c) for c in [80, 53])
ZCQBMJ = "".join(
    chr(c) for c in [67, 111, 110, 102, 105, 103, 78, 117, 109, 98, 101, 114]
)
ZMJIGY = "".join(chr(c) for c in [78, 105, 103, 104, 116])
ZUQEXL = "".join(chr(c) for c in [83, 84, 65, 78, 68, 65, 82, 68])
AIIDNI = [XWAJVD, WAJVDQ, AJVDQL, JVDQLA, VDQLAI, DQLAII, QLAIID, LAIIDN]
AOAWBS = [JVHFTH, ZUQEXL, MAOAWB]
ASSAKQ = [
    JVHFTH,
    VHFTHE,
    HFTHEC,
    FTHECV,
    THECVY,
    HECVYY,
    ECVYYP,
    CVYYPI,
    VYYPIP,
    YYPIPI,
    YPIPIV,
    PIPIVL,
    IPIVLA,
    PIVLAS,
    IVLASS,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    LASSAK,
]
BQSNQL = [XYBQSN, YBQSNQ]
BWJYKL = [SPBWJY, PBWJYK]
CPQIPO = [PIPIVL, UXFEFJ]
FEGZUQ = [JMCBFE, MCBFEG, CBFEGZ, BFEGZU]
FMNHTB = [
    IACQFF,
    ACQFFT,
    CQFFTT,
    QFFTTI,
    FFTTID,
    FTTIDU,
    TTIDUB,
    TIDUBS,
    IDUBSS,
    DUBSSU,
    UBSSUH,
    BSSUHB,
    SSUHBV,
    SUHBVW,
    UHBVWV,
    HBVWVU,
    BVWVUB,
    VWVUBY,
    WVUBYG,
    VUBYGD,
    UBYGDS,
    BYGDSB,
    YGDSBD,
    GDSBDJ,
    DSBDJQ,
    SBDJQR,
    BDJQRJ,
    DJQRJJ,
    JQRJJJ,
    QRJJJV,
    RJJJVY,
    JJJVYF,
    JJVYFC,
    JVYFCR,
    VYFCRT,
    YFCRTF,
    FCRTFM,
    CRTFMN,
    RTFMNH,
    TFMNHT,
]
FWRKIN = [VLASSA, IKFWRK, KFWRKI]
HBSKSO = [
    JVHFTH,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    PIPIVL,
]
IBXTIA = [DNIBXT, NIBXTI]
JEUTOP = [TBJEUT, BJEUTO, VLASSA, VLASSA]
JNIBXY = [JVHFTH, NEJNIB, EJNIBX]
KPHUOJ = [
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    OKPHUO,
]
LJUIKF = [FYLJUI, YLJUIK]
MJIGYO = [IPOUYN, ZMJIGY]
NMHXEK = [QLNMHX, LNMHXE]
OPHUGT = [OKPHUO]
OQNRSJ = [SOOQNR, OOQNRS]
OUYNQJ = [IPOUYN, POUYNQ]
PHUGTY = []
PLSPFT = [IPOUYN, QPLSPF]
QEXLSX = [ZUQEXL, UQEXLS]
QIEFXQ = [
    JVHFTH,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    IVLASS,
]
TACCPQ = [FJTACC, JTACCP]
TOPHUG = [BMJVHF, SSAKQX, AKQXPI, QXPICX, PICXQI, CXQIEF, CTHBSK, KSOKPH]
UJUTYE = [LSXUJU, SXUJUT, XUJUTY]
UNBLKX = [MOUNBL, OUNBLK]
XFEFJT = [JVHFTH, UXFEFJ, PIPIVL]
YEKCWA = [UTYEKC, TYEKCW]
ZILXWA = [VKZILX, KZILXW]


class GeckoConfigStruct:
    def __init__(self, struct_):
        self.struct = struct_

    @property
    def version(self):
        return HUGTYI

    @property
    def output_keys(self):
        return TOPHUG

    @property
    def accessors(self):
        return {
            ZCQBMJ: GeckoByteStructAccessor(self.struct, ZCQBMJ, CQBMJV, QBMJVH),
            BMJVHF: GeckoEnumStructAccessor(
                self.struct, BMJVHF, MJVHFT, None, ASSAKQ, None, None, QBMJVH
            ),
            SSAKQX: GeckoEnumStructAccessor(
                self.struct, SSAKQX, SAKQXP, None, ASSAKQ, None, None, QBMJVH
            ),
            AKQXPI: GeckoEnumStructAccessor(
                self.struct, AKQXPI, KQXPIC, None, ASSAKQ, None, None, QBMJVH
            ),
            QXPICX: GeckoEnumStructAccessor(
                self.struct, QXPICX, XPICXQ, None, ASSAKQ, None, None, QBMJVH
            ),
            PICXQI: GeckoEnumStructAccessor(
                self.struct, PICXQI, ICXQIE, None, ASSAKQ, None, None, QBMJVH
            ),
            CXQIEF: GeckoEnumStructAccessor(
                self.struct, CXQIEF, XQIEFX, None, QIEFXQ, None, None, QBMJVH
            ),
            IEFXQG: GeckoByteStructAccessor(self.struct, IEFXQG, EFXQGL, QBMJVH),
            FXQGLR: GeckoByteStructAccessor(self.struct, FXQGLR, XQGLRA, QBMJVH),
            QGLRAH: GeckoByteStructAccessor(self.struct, QGLRAH, GLRAHE, QBMJVH),
            LRAHEO: GeckoByteStructAccessor(self.struct, LRAHEO, RAHEOC, QBMJVH),
            AHEOCT: GeckoByteStructAccessor(self.struct, AHEOCT, HEOCTH, QBMJVH),
            EOCTHB: GeckoByteStructAccessor(self.struct, EOCTHB, OCTHBS, QBMJVH),
            CTHBSK: GeckoEnumStructAccessor(
                self.struct, CTHBSK, THBSKS, None, HBSKSO, None, None, QBMJVH
            ),
            BSKSOK: GeckoByteStructAccessor(self.struct, BSKSOK, SKSOKP, QBMJVH),
            KSOKPH: GeckoEnumStructAccessor(
                self.struct, KSOKPH, SOKPHU, None, KPHUOJ, None, None, None
            ),
            PHUOJR: GeckoByteStructAccessor(self.struct, PHUOJR, HUOJRJ, None),
            UOJRJH: GeckoByteStructAccessor(self.struct, UOJRJH, OJRJHI, QBMJVH),
            JRJHIU: GeckoByteStructAccessor(self.struct, JRJHIU, RJHIUS, QBMJVH),
            JHIUSO: GeckoByteStructAccessor(self.struct, JHIUSO, HIUSOO, QBMJVH),
            IUSOOQ: GeckoEnumStructAccessor(
                self.struct, IUSOOQ, USOOQN, QNRSJM, OQNRSJ, None, NRSJMC, QBMJVH
            ),
            RSJMCB: GeckoEnumStructAccessor(
                self.struct, RSJMCB, SJMCBF, None, FEGZUQ, None, None, QBMJVH
            ),
            EGZUQE: GeckoEnumStructAccessor(
                self.struct, EGZUQE, GZUQEX, None, QEXLSX, None, None, QBMJVH
            ),
            EXLSXU: GeckoEnumStructAccessor(
                self.struct, EXLSXU, XLSXUJ, None, UJUTYE, None, None, QBMJVH
            ),
            JUTYEK: GeckoEnumStructAccessor(
                self.struct, JUTYEK, USOOQN, NRSJMC, YEKCWA, None, NRSJMC, QBMJVH
            ),
            EKCWAO: GeckoByteStructAccessor(self.struct, EKCWAO, KCWAON, QBMJVH),
            CWAONP: GeckoByteStructAccessor(self.struct, CWAONP, WAONPY, QBMJVH),
            AONPYY: GeckoByteStructAccessor(self.struct, AONPYY, ONPYYL, QBMJVH),
            NPYYLI: GeckoByteStructAccessor(self.struct, NPYYLI, PYYLIU, QBMJVH),
            YYLIUX: GeckoByteStructAccessor(self.struct, YYLIUX, YLIUXF, QBMJVH),
            LIUXFE: GeckoEnumStructAccessor(
                self.struct, LIUXFE, IUXFEF, None, XFEFJT, None, None, QBMJVH
            ),
            FEFJTA: GeckoEnumStructAccessor(
                self.struct, FEFJTA, EFJTAC, None, TACCPQ, None, None, QBMJVH
            ),
            ACCPQI: GeckoEnumStructAccessor(
                self.struct, ACCPQI, CCPQIP, None, CPQIPO, None, None, QBMJVH
            ),
            PQIPOU: GeckoEnumStructAccessor(
                self.struct, PQIPOU, QIPOUY, None, OUYNQJ, None, None, QBMJVH
            ),
            UYNQJY: GeckoByteStructAccessor(self.struct, UYNQJY, YNQJYM, QBMJVH),
            NQJYMO: GeckoTempStructAccessor(self.struct, NQJYMO, QJYMOU, QBMJVH),
            JYMOUN: GeckoEnumStructAccessor(
                self.struct, JYMOUN, YMOUNB, None, UNBLKX, None, None, QBMJVH
            ),
            NBLKXS: GeckoEnumStructAccessor(
                self.struct, NBLKXS, BLKXSJ, None, CPQIPO, None, None, QBMJVH
            ),
            LKXSJW: GeckoByteStructAccessor(self.struct, LKXSJW, KXSJWM, QBMJVH),
            XSJWMN: GeckoTempStructAccessor(self.struct, XSJWMN, SJWMNZ, QBMJVH),
            JWMNZM: GeckoTempStructAccessor(self.struct, JWMNZM, WMNZMJ, QBMJVH),
            MNZMJI: GeckoEnumStructAccessor(
                self.struct, MNZMJI, NZMJIG, None, MJIGYO, None, None, QBMJVH
            ),
            JIGYOU: GeckoByteStructAccessor(self.struct, JIGYOU, IGYOUS, QBMJVH),
            GYOUSP: GeckoByteStructAccessor(self.struct, GYOUSP, YOUSPB, QBMJVH),
            OUSPBW: GeckoEnumStructAccessor(
                self.struct, OUSPBW, USPBWJ, None, BWJYKL, None, None, QBMJVH
            ),
            WJYKLG: GeckoByteStructAccessor(self.struct, WJYKLG, JYKLGQ, QBMJVH),
            YKLGQP: GeckoByteStructAccessor(self.struct, YKLGQP, KLGQPL, QBMJVH),
            LGQPLS: GeckoEnumStructAccessor(
                self.struct, LGQPLS, GQPLSP, None, PLSPFT, None, None, QBMJVH
            ),
            LSPFTS: GeckoByteStructAccessor(self.struct, LSPFTS, SPFTSI, QBMJVH),
            PFTSIF: GeckoTimeStructAccessor(self.struct, PFTSIF, FTSIFJ, QBMJVH),
            TSIFJB: GeckoTimeStructAccessor(self.struct, TSIFJB, SIFJBI, QBMJVH),
            IFJBIA: GeckoTimeStructAccessor(self.struct, IFJBIA, FJBIAM, QBMJVH),
            JBIAMJ: GeckoTimeStructAccessor(self.struct, JBIAMJ, BIAMJM, QBMJVH),
            IAMJMA: GeckoTimeStructAccessor(self.struct, IAMJMA, AMJMAO, QBMJVH),
            MJMAOA: GeckoEnumStructAccessor(
                self.struct, MJMAOA, JMAOAW, None, AOAWBS, None, None, QBMJVH
            ),
            OAWBSI: GeckoBoolStructAccessor(
                self.struct, OAWBSI, AWBSIR, QNRSJM, QBMJVH
            ),
            WBSIRY: GeckoBoolStructAccessor(
                self.struct, WBSIRY, BSIRYX, NRSJMC, QBMJVH
            ),
            SIRYXB: GeckoBoolStructAccessor(
                self.struct, SIRYXB, BSIRYX, QNRSJM, QBMJVH
            ),
            IRYXBQ: GeckoBoolStructAccessor(
                self.struct, IRYXBQ, BSIRYX, RYXBQF, QBMJVH
            ),
            YXBQFY: GeckoBoolStructAccessor(
                self.struct, YXBQFY, BSIRYX, XBQFYL, QBMJVH
            ),
            BQFYLJ: GeckoEnumStructAccessor(
                self.struct, BQFYLJ, QFYLJU, None, LJUIKF, None, None, QBMJVH
            ),
            JUIKFW: GeckoEnumStructAccessor(
                self.struct, JUIKFW, UIKFWR, None, FWRKIN, None, None, QBMJVH
            ),
            WRKINE: GeckoByteStructAccessor(self.struct, WRKINE, RKINEJ, QBMJVH),
            KINEJN: GeckoEnumStructAccessor(
                self.struct, KINEJN, INEJNI, None, JNIBXY, None, None, QBMJVH
            ),
            NIBXYB: GeckoWordStructAccessor(self.struct, NIBXYB, IBXYBQ, QBMJVH),
            BXYBQS: GeckoEnumStructAccessor(
                self.struct, BXYBQS, USOOQN, RYXBQF, BQSNQL, None, NRSJMC, QBMJVH
            ),
            QSNQLN: GeckoBoolStructAccessor(
                self.struct, QSNQLN, USOOQN, SNQLNM, QBMJVH
            ),
            NQLNMH: GeckoEnumStructAccessor(
                self.struct, NQLNMH, USOOQN, MHXEKV, NMHXEK, None, NRSJMC, QBMJVH
            ),
            HXEKVK: GeckoBoolStructAccessor(
                self.struct, HXEKVK, XEKVKZ, QNRSJM, QBMJVH
            ),
            EKVKZI: GeckoBoolStructAccessor(
                self.struct, EKVKZI, XEKVKZ, RYXBQF, QBMJVH
            ),
            KVKZIL: GeckoEnumStructAccessor(
                self.struct, KVKZIL, XEKVKZ, NRSJMC, ZILXWA, None, NRSJMC, QBMJVH
            ),
            ILXWAJ: GeckoEnumStructAccessor(
                self.struct, ILXWAJ, XEKVKZ, XBQFYL, ZILXWA, None, NRSJMC, QBMJVH
            ),
            LXWAJV: GeckoEnumStructAccessor(
                self.struct, LXWAJV, XEKVKZ, SNQLNM, AIIDNI, None, IIDNIB, QBMJVH
            ),
            IDNIBX: GeckoEnumStructAccessor(
                self.struct, IDNIBX, XEKVKZ, BXTIAC, IBXTIA, None, NRSJMC, QBMJVH
            ),
            MNHTBJ: GeckoBoolStructAccessor(
                self.struct, MNHTBJ, NHTBJE, QNRSJM, QBMJVH
            ),
            HTBJEU: GeckoEnumStructAccessor(
                self.struct, HTBJEU, NHTBJE, NRSJMC, JEUTOP, None, SNQLNM, QBMJVH
            ),
            EUTOPH: GeckoByteStructAccessor(self.struct, EUTOPH, UTOPHU, QBMJVH),
        }



""" Gecko automation support for sensors """

from .base import GeckoAutomationFacadeBase


class GeckoSensorBase(GeckoAutomationFacadeBase):
    """Base sensor allows non-accessor sensors to be implemented"""

    def __init__(self, facade, name, device_class=None):
        super().__init__(facade, name, name.upper())
        self._device_class = device_class

    @property
    def state(self):
        """The state of the sensor"""
        return None

    @property
    def unit_of_measurement(self):
        """The unit of measurement for the sensor, or None"""
        return None

    @property
    def device_class(self):
        """The device class"""
        return self._device_class

    def __repr__(self):
        return f"{self.name}: {self.state}"


########################################################################################
class GeckoSensor(GeckoSensorBase):
    """Sensors wrap accessors state with extra units and device
    class properties"""

    def __init__(self, facade, name, accessor, unit_accessor=None, device_class=None):
        super().__init__(facade, name, device_class)
        self._accessor = accessor
        # Bubble up change notification
        accessor.watch(self._on_change)
        self._unit_of_measurement_accessor = unit_accessor
        if unit_accessor:
            unit_accessor.watch(self._on_change)

    @property
    def state(self):
        """The state of the sensor"""
        return self._accessor.value

    @property
    def unit_of_measurement(self):
        """The unit of measurement for the sensor, or None"""
        if self._unit_of_measurement_accessor is None:
            return None
        return self._unit_of_measurement_accessor.value

    @property
    def accessor(self):
        """Access the accessor member"""
        return self._accessor

    @property
    def monitor(self):
        return f"{self.accessor.tag}: {self.state}"


########################################################################################
class GeckoBinarySensor(GeckoSensor):
    """Binary sensors only have two states"""

    @property
    def is_on(self):
        """Determine if the sensor is on or not"""
        state = self.state
        if isinstance(state, bool):
            return state
        if state == "":
            return False
        return state != "OFF"
